// Copyright 2023 The Manifold Authors.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Note that this file is shared between the standard polygon test and fuzzer
// seed.

/**
 * These polygons are all valid geometry. Some are clearly valid, while many are
 * marginal, but all should produce correct topology and geometry, within
 * tolerance.
 */
TEST(Polygon, SimpleHole) {
  Polygons polys;
  polys.push_back({
      {0, -2},  //
      {2, 2},   //
      {0, 4},   //
      {-3, 3},  //
  });
  polys.push_back({
      {0, -1},  //
      {-1, 1},  //
      {1, 1},   //
  });
  TestPoly(polys, 7);
}

TEST(Polygon, SimpleHole2) {
  Polygons polys;
  polys.push_back({
      {0, 1.63299},           //
      {-1.41421, -0.816496},  //
      {1.41421, -0.816496},   //
  });
  polys.push_back({
      {0, 1.02062},           //
      {0.883883, -0.51031},   //
      {-0.883883, -0.51031},  //
  });
  TestPoly(polys, 6);
}

TEST(Polygon, MultiMerge) {
  Polygons polys;
  polys.push_back({
      {-7, 0},   //
      {-6, 3},   //
      {-5, 1},   //
      {-4, 6},   //
      {-3, 2},   //
      {-2, 5},   //
      {-1, 4},   //
      {0, 12},   //
      {-6, 10},  //
      {-8, 11},  //
  });
  polys.push_back({
      {-5, 7},  //
      {-6, 8},  //
      {-5, 9},  //
  });
  TestPoly(polys, 13);
}

TEST(Polygon, SimpleHoles) {
  Polygons polys;
  polys.push_back({
      {-0.277777791, 0.376543224},  //
      {0.5, -0.233333275},          //
      {0.285185158, 0.214814827},   //
  });
  polys.push_back({
      {0.325617313, -0.055555556},  //
      {0.277777791, 0.092592679},   //
      {0.328042328, 0.0608465709},  //
  });
  polys.push_back({
      {0.212963, 0.092592597},     //
      {0.203703716, 0.141975299},  //
      {0.212962985, 0.138888896},  //
  });
  TestPoly(polys, 11);
}

TEST(Polygon, SimpleHoles2) {
  Polygons polys;
  polys.push_back({
      {-0.0185185187, 0.5},         //
      {-0.5, 0.49253732},           //
      {0.166666672, 0.0370370671},  //
      {0.0617283992, 0.438271612},  //
  });
  polys.push_back({
      {-0.179012358, 0.448559672},  //
      {-0.179012358, 0.438271612},  //
      {-0.188271582, 0.438271612},  //
  });
  polys.push_back({
      {-0.413580239, 0.45679009},   //
      {-0.401234567, 0.448559672},  //
      {-0.401234567, 0.438271612},  //
  });
  polys.push_back({
      {0.0308641978, 0.450617284},  //
      {0.0432098769, 0.450617284},  //
      {0.0493827164, 0.438271612},  //
  });
  TestPoly(polys, 17);
}

TEST(Polygon, Looping1) {
  Polygons polys;
  polys.push_back({
      {0.f, -2.f},  //
      {2.f, 2.f},   //
      {0.f, 4.f},   //
      {-3.f, 3.f},  //
  });
  polys.push_back({
      {1.40282347e+38f, 1.40282347e+38f},  //
      {-0.f, 0.f},                         //
      {0.f, 1.f},                          //
      {1.f, 0.f},                          //
  });
  polys.push_back({
      {0.f, -1.f},  //
      {-1.f, 1.f},  //
      {1.f, 1.f},   //
  });
  TestPoly(polys, 5);
}

TEST(Polygon, Looping2) {
  Polygons polys;
  polys.push_back({
      {-0.277777791f, 0.376543224f},  //
      {0.5f, -0.233333275f},          //
      {0.285185158f, 0.214814827f},   //
  });
  polys.push_back({
      {0.325617313f, -0.055555556f},  //
      {0.277777791f, 0.092592679f},   //
      {0.328042328f, 0.0608465709f},  //
  });
  polys.push_back({
      {3.40282347e+38f, 0.283553332f},  //
      {0.212963f, 0.092592597f},        //
      {0.203703716f, 0.141975299f},     //
      {0.212962985f, 0.138888896f},     //
  });
  TestPoly(polys, 4);
}

TEST(Polygon, AlignedHoles) {
  Polygons polys;
  polys.push_back({
      {-0.391621143, 0.5},         //
      {0.5, -0.305555522},         //
      {0.5, -0.0629629567},        //
      {0.214285731, 0.285714298},  //
  });
  polys.push_back({
      {-0.174897119, 0.403292179},  //
      {-0.179012358, 0.395061731},  //
      {-0.19135803, 0.407407403},   //
  });
  polys.push_back({
      {0.325617313, -0.055555556},  //
      {0.277777791, -0.055555556},  //
      {0.314176232, 0.0747126788},  //
  });
  polys.push_back({
      {0.339506179, -0.0740740374},  //
      {0.339506179, -0.0833333284},  //
      {0.327160507, -0.0802469105},  //
  });
  polys.push_back({
      {0.364197522, -0.129629627},   //
      {0.314814836, -0.0802469477},  //
      {0.351851851, -0.092592597},   //
  });
  polys.push_back({
      {0.302469134, -0.0679012388},  //
      {0.30864194, -0.0802469105},   //
      {0.302469134, -0.0833332911},  //
  });
  TestPoly(polys, 27);
}

TEST(Polygon, Outers) {
  Polygons polys;
  polys.push_back({
      {-4.95815706, 4.55738497},  //
      {-4.85924578, 4.65629625},  //
      {-4.85924578, 4.65629625},  //
      {-4.91438103, 4.64493704},  //
      {-4.91438103, 4.64493704},  //
      {-4.95815706, 4.65629625},  //
      {-4.95815706, 4.65629625},  //
      {-4.95815706, 4.65629625},  //
  });
  polys.push_back({
      {-4.95815706, 4.95815706},  //
      {-4.95815706, 4.85725117},  //
      {-4.95815706, 4.85725117},  //
      {-4.95815706, 4.85725117},  //
      {-4.95815706, 4.95815706},  //
      {-4.90248775, 4.9224453},   //
      {-4.87105799, 4.95158386},  //
      {-4.87105799, 4.95158386},  //
      {-4.85925102, 4.95815706},  //
  });
  polys.push_back({
      {-4.55738497, 4.95815706},  //
      {-4.65629101, 4.95815706},  //
      {-4.65105772, 4.95158339},  //
      {-4.64647388, 4.89330769},  //
      {-4.64647388, 4.89330769},  //
      {-4.65829086, 4.85725117},  //
      {-4.65829086, 4.85725117},  //
  });
  TestPoly(polys, 18);
}

TEST(Polygon, Holes) {
  Polygons polys;
  polys.push_back({
      {0.462962955, -0.5},          //
      {0.462962955, 0.129629627},   //
      {0.432784647, -0.314814836},  //
      {0.462962955, -0.316186577},  //
      {0.461591214, -0.351851851},  //
      {0.431412905, -0.351851851},  //
      {0.428669423, -0.42592594},   //
      {0.462962955, -0.427297682},  //
      {0.42592594, -0.5},           //
  });
  polys.push_back({
      {0.438271612, -0.413122982},  //
      {0.438728869, -0.401234567},  //
      {0.450617284, -0.401691824},  //
  });
  polys.push_back({
      {0.438271612, -0.302011877},  //
      {0.438728869, -0.290123463},  //
      {0.450617284, -0.29058072},   //
  });
  TestPoly(polys, 17);
}

TEST(Polygon, Holes2) {
  Polygons polys;
  polys.push_back({
      {31.5, -40.5},        //
      {31.5, 24.5602417},   //
      {29.4397583, 22.5},   //
      {28.5, 7.5},          //
      {24.5602417, -22.5},  //
      {31.5, -31.5},        //
      {23.5843372, -31.5},  //
  });
  polys.push_back({
      {30.5, -6.27108431},  //
      {29.5, -5.72891569},  //
      {30.2710838, -5.5},   //
  });
  polys.push_back({
      {30.5, 18.5},  //
      {29.5, 17.5},  //
      {29.5, 18.5},  //
  });
  polys.push_back({
      {30.5, 20.7289162},  //
      {29.5, 21.2710838},  //
      {30.2710838, 21.5},  //
  });
  TestPoly(polys, 20);
};

TEST(Polygon, Holes3) {
  Polygons polys;
  polys.push_back({
      {0.154320985, 0.438271612},   //
      {0.0802469105, 0.216049403},  //
      {0.0555555634, 0.166666672},  //
      {-0.166666672, -0.5},         //
      {0.166666672, -0.5},          //
  });
  polys.push_back({
      {0.141975313, -0.187242806},  //
      {0.146090537, -0.179012358},  //
      {0.154320985, -0.183127582},  //
  });
  polys.push_back({
      {-0.00617283955, -0.187242806},  //
      {-0.00205761334, -0.179012358},  //
      {0.00617283955, -0.183127582},   //
  });
  polys.push_back({
      {0.0679012388, 0.19135803},   //
      {0.0802469105, 0.19135803},   //
      {0.0802469105, 0.179012358},  //
  });
  polys.push_back({
      {0.104938269, 0.257201672},  //
      {0.104938269, 0.265432119},  //
      {0.117283955, 0.265432119},  //
  });
  TestPoly(polys, 23);
};

TEST(Polygon, Holes4) {
  Polygons polys;
  polys.push_back({
      {0.166666672, 0.5},           //
      {0.129629627, 0.314814836},   //
      {0.0802469105, 0.216049403},  //
      {0.0555555634, 0.166666672},  //
      {0.166666672, 0.0555555634},  //
      {-0.166666672, -0.5},         //
      {0.166666672, -0.5},          //
  });
  polys.push_back({
      {0.1090535, 0.19135803},     //
      {0.117283955, 0.187242806},  //
      {0.113168724, 0.179012358},  //
  });
  polys.push_back({
      {0.0679012388, 0.183127582},  //
      {0.0679012388, 0.19135803},   //
      {0.0802469105, 0.179012358},  //
  });
  polys.push_back({
      {0.104938269, 0.257201672},  //
      {0.104938269, 0.265432119},  //
      {0.117283955, 0.265432119},  //
  });
  TestPoly(polys, 20);
};

TEST(Polygon, SmallHole) {
  Polygons polys;
  polys.push_back({
      {8.93379593, -41.2573624},  //
      {10.5178413, -41.2440758},  //
      {8.99354267, -41.1997375},  //
  });
  polys.push_back({
      {9.08317471, -41.2453728},  //
      {9.08610725, -41.242527},   //
      {9.0890007, -41.2397537},   //
      {9.08943081, -41.2436409},  //
      {9.08946514, -41.2439499},  //
      {9.08924961, -41.2441826},  //
      {9.0890007, -41.2444496},   //
  });
  TestPoly(polys, 10);
};

TEST(Polygon, Precision4) {
  Polygons polys;
  polys.push_back({
      {-7.57671118, -12.7964983},  //
      {-7.57698059, -12.796319},   //
      {-7.57698631, -12.7967968},  //
      {-7.63940859, -12.9102335},  //
      {-7.58357, -12.7919359},     //
      {-7.58663845, -12.7898951},  //
      {-7.63940859, -12.9102335},  //
  });
  TestPoly(polys, 5, 0.0004);
};

TEST(Polygon, CoincidentHole) {
  Polygons polys;
  polys.push_back({
      {9.82835007, -0.939956188},  //
      {10.1291866, -1.12029469},   //
      {10.1291866, -0.759618163},  //
  });
  polys.push_back({
      {9.82835007, -0.939956307},  //
      {10.053978, -0.80470264},    //
      {9.97876835, -1.0301255},    //
  });
  TestPoly(polys, 6);
};

TEST(Polygon, Holes5) {
  Polygons polys;
  polys.push_back({
      {-0.166666672, -0.166666672},  //
      {-0.462962955, 0.42592594},    //
      {-0.5, -0.5},                  //
  });
  polys.push_back({
      {-0.240740761, -0.129629627},  //
      {-0.203703716, -0.092592597},  //
      {-0.203703716, -0.092592597},  //
      {-0.203703716, -0.129629627},  //
  });
  polys.push_back({
      {-0.388888896, -0.055555556},  //
      {-0.277777791, 0.055555556},   //
      {-0.277777791, -0.055555556},  //
  });
  TestPoly(polys, 12);
};

TEST(Polygon, Holes6) {
  Polygons polys;
  polys.push_back({
      {0.388888896, 0.277777791},  //
      {0.277777791, 0.388888896},  //
      {0.388888896, 0.388888896},  //
  });
  polys.push_back({
      {0.055555556, 0.388888896},   //
      {-0.055555556, 0.277777791},  //
      {-0.055555556, 0.388888896},  //
  });
  polys.push_back({
      {-0.277777791, 0.388888896},  //
      {-0.388888896, 0.277777791},  //
      {-0.388888896, 0.388888896},  //
  });
  polys.push_back({
      {-0.5, -0.5},  //
      {0.5, -0.5},   //
      {0.5, 0.5},    //
      {-0.5, 0.5},   //
  });
  TestPoly(polys, 17);
};

TEST(Polygon, Holes7) {
  Polygons polys;
  polys.push_back({
      {0.203703731, -0.203703716},  //
      {-0.388888896, 0.277777791},  //
      {-0.5, -0.5},                 //
  });
  polys.push_back({
      {-0.129629627, -0.203703716},  //
      {-0.092592597, -0.203703716},  //
      {-0.092592597, -0.240740761},  //
  });
  polys.push_back({
      {-0.351851851, -0.203703716},  //
      {-0.314814836, -0.203703716},  //
      {-0.314814836, -0.240740761},  //
  });
  TestPoly(polys, 11);
};

TEST(Polygon, Holes8) {
  Polygons polys;
  polys.push_back({
      {-0.166666672, 0.166666672},  //
      {-0.42592594, 0.42592594},    //
      {-0.5, -0.5},                 //
  });
  polys.push_back({
      {-0.277777791, -0.055555556},  //
      {-0.388888896, 0.055555556},   //
      {-0.277777791, 0.055555556},   //
      {-0.277777791, -0.055555556},  //
  });
  polys.push_back({
      {-0.462962955, -0.129629627},  //
      {-0.42592594, -0.092592597},   //
      {-0.42592594, -0.129629627},   //
  });
  TestPoly(polys, 12);
};

TEST(Polygon, Holes9) {
  Polygons polys;
  polys.push_back({
      {-0.166666672, -0.166666672},  //
      {-0.166666672, 0.166666672},   //
      {-0.166666657, 0.166666672},   //
      {-0.462962955, 0.42592594},    //
      {-0.5, -0.5},                  //
  });
  polys.push_back({
      {-0.277777791, -0.055555556},  //
      {-0.277777791, -0.055555556},  //
      {-0.388888896, 0.055555556},   //
      {-0.277777791, 0.055555556},   //
  });
  polys.push_back({
      {-0.462962955, -0.129629627},  //
      {-0.42592594, -0.092592597},   //
      {-0.42592594, -0.092592597},   //
      {-0.42592594, -0.129629627},   //
  });
  TestPoly(polys, 15);
};

TEST(Polygon, Eberly) {
  Polygons polys;
  polys.push_back({
      {0, 0},    //
      {9, 0},    //
      {7, -1},   //
      {8, -1},   //
      {10, -5},  //
      {10, 3},   //
      {0, 3},    //
  });
  polys.push_back({
      {1, 1},  //
      {1, 2},  //
      {2, 1},  //
  });
  TestPoly(polys, 10);
};

TEST(Polygon, Ugly) {
  Polygons polys;
  polys.push_back({
      {2379.21118, 13009.6982},  //
      {2292.45776, 13566.7129},  //
      {2048.95142, 13894.6953},  //
      {2054.84497, 13878.9023},  //
      {2055.11914, 13878.9707},  //
      {2186.24487, 13527.4189},  //
      {2186.24487, 13527.4189},  //
      {2186.24487, 13527.4189},  //
      {2189.47729, 13552.7422},  //
      {2208.28711, 13577.457},   //
      {2208.28711, 13577.457},   //
      {2236.93433, 13589.4551},  //
      {2236.93433, 13589.4551},  //
      {2236.93433, 13589.4551},  //
      {2208.28711, 13577.457},   //
      {2292.45703, 13566.7129},  //
      {2208.28711, 13577.457},   //
      {2208.28711, 13577.457},   //
      {2189.47729, 13552.7422},  //
      {2189.47729, 13552.7422},  //
      {2189.47729, 13552.7422},  //
      {2222.25732, 13474.4775},  //
      {2222.25732, 13474.4775},  //
      {2222.25732, 13474.4775},  //
      {2222.25732, 13474.4775},  //
      {2222.25732, 13474.4775},  //
      {2189.47729, 13552.7422},  //
      {2186.24487, 13527.4189},  //
      {2186.61182, 13525.8066},  //
      {2222.38232, 13429.9521},  //
      {2222.38232, 13429.9521},  //
      {2222.38232, 13429.9521},  //
  });
  polys.push_back({
      {2379.03198, 13010.5518},  //
      {2378.89648, 13010.541},   //
      {2289.28784, 13251.1582},  //
      {2379.03027, 13010.5566},  //
  });
  TestPoly(polys, 36);
};

TEST(Polygon, Sponge4a) {
  Polygons polys;
  polys.push_back({
      {0.166666672, -0.0555555671},  //
      {0.216049403, -0.0720164701},  //
      {0.216049403, -0.0679012388},  //
      {0.220164627, -0.0679012388},  //
      {0.228395075, -0.0679012388},  //
      {0.228395075, -0.0720164627},  //
      {0.228395075, -0.076131694},   //
      {0.314814836, -0.104938284},   //
      {0.314814836, -0.092592597},   //
      {0.327160507, -0.092592597},   //
      {0.351851851, -0.092592597},   //
      {0.351851851, -0.104938276},   //
      {0.351851851, -0.117283955},   //
      {0.438271612, -0.146090537},   //
      {0.438271612, -0.141975313},   //
      {0.442386836, -0.141975313},   //
      {0.450617284, -0.141975313},   //
      {0.376543224, -0.104938284},   //
      {0.376543224, -0.1090535},     //
      {0.376543224, -0.117283955},   //
      {0.372428, -0.117283955},      //
      {0.364197552, -0.117283955},   //
      {0.364197552, -0.113168724},   //
      {0.364197552, -0.104938269},   //
      {0.368312776, -0.104938269},   //
      {0.376543194, -0.104938269},   //
      {0.302469134, -0.0679012388},  //
      {0.302469134, -0.0720164627},  //
      {0.302469134, -0.0802469105},  //
      {0.29835391, -0.0802469105},   //
      {0.290123463, -0.0802469105},  //
      {0.290123463, -0.0761316866},  //
      {0.290123463, -0.0679012388},  //
      {0.294238687, -0.0679012388},  //
      {0.302469134, -0.0679012388},  //
      {0.166666672, 0},              //
  });
  polys.push_back({
      {0.179012358, -0.0432098769},  //
      {0.179012358, -0.0390946493},  //
      {0.179012358, -0.0308641978},  //
      {0.183127582, -0.0308641978},  //
      {0.19135803, -0.0308641978},   //
      {0.19135803, -0.0349794254},   //
      {0.19135803, -0.0432098769},   //
      {0.187242806, -0.0432098769},  //
  });
  polys.push_back({
      {0.216049403, -0.0432098769},  //
      {0.216049403, -0.0390946493},  //
      {0.216049403, -0.0308641978},  //
      {0.220164627, -0.0308641978},  //
      {0.228395075, -0.0308641978},  //
      {0.228395075, -0.0349794254},  //
      {0.228395075, -0.0432098769},  //
      {0.224279851, -0.0432098769},  //
  });
  polys.push_back({
      {0.253086448, -0.0802469105},  //
      {0.253086448, -0.0761316866},  //
      {0.253086448, -0.0679012388},  //
      {0.257201672, -0.0679012388},  //
      {0.265432119, -0.0679012388},  //
      {0.265432119, -0.0720164627},  //
      {0.265432119, -0.0802469105},  //
      {0.261316895, -0.0802469105},  //
  });
  TestPoly(polys, 64);
};

TEST(Polygon, CoincidentHole2) {
  Polygons polys;
  polys.push_back({
      {-6.53556156, 12.4571409},  //
      {-6.72912979, 12.5610456},  //
      {-6.7918272, 12.5019674},   //
  });
  polys.push_back({
      {-6.72903824, 12.5008945},  //
      {-6.72912979, 12.5008078},  //
      {-6.72931194, 12.5008106},  //
      {-6.7918272, 12.5019674},   //
      {-6.72912979, 12.5610456},  //
      {-6.72894907, 12.501153},   //
      {-6.72894812, 12.5009804},  //
  });
  TestPoly(polys, 10, 0.0005);
};

TEST(Polygon, CoincidentHole3) {
  Polygons polys;
  polys.push_back({
      {9.76247501, -8.75089836},  //
      {9.78284931, -8.75129509},  //
      {9.78242302, -8.75128555},  //
      {9.76939392, -8.74437904},  //
      {9.76939392, -8.74437904},  //
  });
  polys.push_back({
      {9.76247501, -8.75089836},  //
      {9.76939392, -8.74437904},  //
      {9.77548981, -8.75115108},  //
      {9.77501488, -8.7511425},   //
  });
  TestPoly(polys, 9, 0.0002);
};

TEST(Polygon, CoincidentHole4) {
  Polygons polys;
  polys.push_back({
      {1.44564819, 9.71884632},  //
      {1.48651409, 9.71884632},  //
      {1.44735146, 9.76247501},  //
  });
  polys.push_back({
      {1.4456495, 9.71884632},   //
      {1.44564915, 9.71884632},  //
      {1.44564879, 9.71884632},  //
      {1.44564819, 9.71884632},  //
      {1.44586205, 9.72430038},  //
      {1.44992185, 9.72319221},  //
      {1.44936395, 9.71884632},  //
  });
  TestPoly(polys, 10, 0.0002);
};

TEST(Polygon, Colinear) {
  Polygons polys;
  polys.push_back({
      {-5.48368, -3.73905},   //
      {-4.9881, -4.51552},    //
      {-4.78988, -4.13186},   //
      {-4.82012, -4.13999},   //
      {-4.84314, -4.14617},   //
      {-4.85738, -4.13581},   //
      {-4.86772, -4.12831},   //
      {-4.87337, -4.12422},   //
      {-4.88097, -4.1187},    //
      {-4.89799, -4.10634},   //
      {-4.90219, -4.103290},  //
      {-4.90826, -4.098871},  //
      {-4.90846, -4.098732},  //
      {-4.91227, -4.095973},  //
      {-4.92199, -4.08894},   //
      {-5.0245, -4.014435},   //
      {-5.02494, -4.014126},  //
      {-5.02536, -4.013817},  //
      {-5.0316, -4.009278},   //
      {-5.03211, -4.008919},  //
      {-5.05197, -3.994480},  //
      {-5.14757, -3.925041},  //
      {-5.21287, -3.87762},   //
      {-5.29419, -3.818533},  //
      {-5.29907, -3.814994},  //
      {-5.36732, -3.765415},  //
  });
  TestPoly(polys, 24);
}

TEST(Polygon, Colinear4) {
  Polygons polys;
  polys.push_back({
      {-22.7753239, 8.03145695},  //
      {-22.8791199, 8.04628658},  //
      {-22.8791199, 8.0424366},   //
      {-22.8792648, 8.04237938},  //
      {-22.8794041, 8.04239368},  //
      {-22.9786491, 8.05282307},  //
      {-22.9923439, 8.06109047},  //
      {-22.995327, 8.06289101},   //
      {-23.0484314, 8.07047844},  //
      {-23.056942, 8.06713009},   //
      {-23.0641346, 8.06429958},  //
      {-23.0679455, 8.06355},     //
      {-23.0689487, 8.06335258},  //
      {-23.0691357, 8.06331635},  //
      {-23.0723953, 8.06429958},  //
      {-23.1114883, 8.0794878},   //
      {-24.4342442, 8.26848221},  //
      {-24.4342442, 8.2057848},   //
  });
  TestPoly(polys, 16, 0.0004);
}

TEST(Polygon, Merges) {
  Polygons polys;
  polys.push_back({
      {-3.22039, 10.2769},   //
      {-3.12437, 10.4147},   //
      {-3.99093, 10.1781},   //
      {-3.8154, 10.0716},    //
      {-3.78982, 10.0893},   //
      {-3.55033, 10.2558},   //
      {-3.50073, 10.2549},   //
      {-3.47018, 10.2572},   //
      {-3.42633, 10.2605},   //
      {-3.34311, 10.2604},   //
      {-3.32096, 10.26330},  //
  });
  TestPoly(polys, 9);
}

TEST(Polygon, ExtraTriangle) {
  Polygons polys;
  polys.push_back({
      {1.23141634, -0.493547261},  //
      {1.23142254, -0.493540883},  //
      {1.23088336, -0.457464248},  //
  });
  polys.push_back({
      {1.23146737, -0.493494928},   //
      {1.47253549, -0.24623163},    //
      {1.47253144, -0.246230021},   //
      {1.47166216, -0.246238187},   //
      {1.46963537, -0.24623026},    //
      {1.46811843, -0.246224999},   //
      {1.46594918, -0.246223733},   //
      {1.46594965, -0.2462321670},  //
      {1.46594965, -0.2462322411},  //
      {1.46594083, -0.2462237332},  //
      {1.46591508, -0.2462237183},  //
      {1.46594179, -0.2462314374},  //
      {1.46585774, -0.2462236885},  //
      {1.46577716, -0.2462236296},  //
      {1.46590662, -0.2462312137},  //
      {1.46550393, -0.2462234658},  //
      {1.46081161, -0.2462207389},  //
      {1.46407437, -0.2462289040},  //
      {1.45568836, -0.2462277561},  //
      {1.42199826, -0.2602537572},  //
      {1.40801644, -0.2462047343},  //
      {1.40688479, -0.246203084},   //
      {1.38571239, -0.2461840815},  //
      {1.30817795, -0.2461582426},  //
      {1.28997684, -0.3280568727},  //
      {1.30974603, -0.3150095348},  //
      {1.31756043, -0.3649175179},  //
  });
  TestPoly(polys, 26);
}

TEST(Polygon, SpongeThin) {
  Polygons polys;
  polys.push_back({
      {-0.5, -0.4753086571},          //
      {-0.487654328, -0.4754610662},  //
      {-0.487654328, -0.4753086573},  //
      {-0.487654328, -0.4753086574},  //
      {-0.487654328, -0.4753086575},  //
      {-0.475308657, -0.4753086576},  //
      {-0.475308657, -0.4756134757},  //
      {-0.462962985, -0.4757659148},  //
      {-0.462962985, -0.4753086579},  //
  });
  polys.push_back({
      {0.42592591, -0.4753086570},   //
      {0.351851851, -0.4753086571},  //
      {0.351851851, -0.4753086572},  //
      {0.351851851, -0.485825333},   //
      {0.364197552, -0.4859777694},  //
      {0.364197552, -0.4753086575},  //
      {0.364197552, -0.4753086576},  //
      {0.364197552, -0.4753086577},  //
      {0.376543224, -0.4753086578},  //
      {0.376543224, -0.4861301789},  //
      {0.401234567, -0.4864349960},  //
      {0.401234567, -0.4753086571},  //
      {0.401234567, -0.4753086572},  //
      {0.401234567, -0.4753086573},  //
      {0.413580239, -0.4753086574},  //
      {0.413580239, -0.4865874055},  //
      {0.42592591, -0.4867398446},   //
  });
  polys.push_back({
      {0.314814806, -0.4753086577},  //
      {0.240740761, -0.4753086578},  //
      {0.240740761, -0.4753086579},  //
      {0.240740761, -0.4844535890},  //
      {0.253086448, -0.4846060281},  //
      {0.253086448, -0.4753086572},  //
      {0.253086448, -0.4753086573},  //
      {0.253086448, -0.4753086574},  //
      {0.265432119, -0.4753086575},  //
      {0.265432119, -0.4847584376},  //
      {0.290123463, -0.4850632557},  //
      {0.290123463, -0.4753086578},  //
      {0.290123463, -0.4753086579},  //
      {0.290123463, -0.4753086570},  //
      {0.302469134, -0.4753086571},  //
      {0.302469134, -0.4852156642},  //
      {0.314814806, -0.4853681033},  //
      {0.314814806, -0.4753086574},  //
  });
  TestPoly(polys, 38);
}

TEST(Polygon, ColinearY) {
  Polygons polys;
  polys.push_back({
      {0, 0},   //
      {1, 1},   //
      {2, 1},   //
      {3, 1},   //
      {4, 1},   //
      {4, 2},   //
      {3, 2},   //
      {2, 2},   //
      {1, 2},   //
      {0, 3},   //
      {-1, 2},  //
      {-2, 2},  //
      {-3, 2},  //
      {-4, 2},  //
      {-4, 1},  //
      {-3, 1},  //
      {-2, 1},  //
      {-1, 1},  //
  });
  TestPoly(polys, 16);
}

TEST(Polygon, NearlyColinearY) {
  Polygons polys;
  polys.push_back({
      {3.5, 0},       //
      {3.0, 0.5},     //
      {3.0, 0.0144},  //
      {2.0, 0.4},     //
      {1.8, 0.1},     //
      {1.6, 0.0155},  //
      {1.5, 0.0147},  //
      {1.4, 0.0144},  //
      {1.0, 0.1},     //
      {0, 0},         //
  });
  TestPoly(polys, 8, 0.001);
}

TEST(Polygon, Concave) {
  Polygons polys;
  polys.push_back({
      {-0.707107008, -0.707107008},    //
      {1, 0},                          //
      {0.683013022, 0},                //
      {0.379409999, -0.232962996},     //
      {0.379409999, -0.232962996},     //
      {1.49012003e-08, -0.183013007},  //
      {1.49012003e-08, -0.183013007},  //
      {-0.140431002, 0},               //
      {-1, 0},                         //
  });
  TestPoly(polys, 7);
}

TEST(Polygon, Concave2) {
  Polygons polys;
  polys.push_back({
      {4, 0},   //
      {3, 2},   //
      {3, 3},   //
      {6, 5},   //
      {6, 14},  //
      {0, 13},  //
      {0, 12},  //
      {3, 11},  //
      {4, 10},  //
      {5, 8},   //
      {1, 7},   //
      {2, 1},   //
  });
  TestPoly(polys, 10);
}

TEST(Polygon, Sliver) {
  Polygons polys;
  polys.push_back({
      {2.82002997, 0},             //
      {2.82002997, 0},             //
      {2.06105995, 0},             //
      {2.05792999, 0.0680378973},  //
      {2.06410003, 0.206908002},   //
      {2.28446007, 1.04696},       //
      {2.35005999, 1.24989998},    //
      {-2.82002997, 15},           //
      {-2.82002997, 0},            //
  });
  TestPoly(polys, 7);
}

TEST(Polygon, Duplicate) {
  Polygons polys;
  polys.push_back({
      {-32.0774002, -10.4309998},   //
      {-31.7346992, -6.10348988},   //
      {-31.8645992, -5.61858988},   //
      {-31.8645992, -5.61858988},   //
      {-31.8645992, -5.61858988},   //
      {-31.8645992, -5.61858988},   //
      {-31.8645992, -5.61858988},   //
      {-31.8645992, -5.61858988},   //
      {-31.8645992, -5.61858988},   //
      {-31.8645992, -5.61858988},   //
      {-31.8645992, -5.618589880},  //
      {-31.8645992, -5.618589881},  //
      {-31.8645992, -5.618589882},  //
      {-31.8645992, -5.618589883},  //
      {-31.8645992, -5.618589884},  //
      {-31.8645992, -5.618589885},  //
      {-31.8645992, -5.618589886},  //
      {-31.8645992, -5.618589887},  //
      {-31.8645992, -5.618589888},  //
      {-31.8645992, -5.618589889},  //
      {-31.8645992, -5.618589880},  //
      {-32.0774002, -3.18654991},   //
  });
  TestPoly(polys, 20);
}

TEST(Polygon, Duplicate2) {
  Polygons polys;
  polys.push_back({
      {0, 0},   //
      {0, -1},  //
      {1, 1},   //
      {-1, 0},  //
      {0, 0},   //

  });
  TestPoly(polys, 3);
}

TEST(Polygon, Folded) {
  Polygons polys;
  polys.push_back({
      {2.82002997, 0},              //
      {1.23706996, 4.20994997},     //
      {1.14140999, 4.09090996},     //
      {1.05895996, 3.94496012},     //
      {0.00757742021, 2.72726989},  //
      {-0.468091995, 1.94363999},   //
      {-1.06106997, 1.36363995},    //
      {-1.79214001, 0.346489996},   //
      {-2.27416992, 0},             //
      {-2.82002997, 0},             //
      {-2.82002997, 0},             //
  });
  TestPoly(polys, 9);
}

TEST(Polygon, NearlyLinear) {
  Polygons polys;
  polys.push_back({
      {2.82002997, -8.22814036e-05},    //
      {2.82002997, -8.22814036e-05},    //
      {2.31802011, -8.22814036e-05},    //
      {-0.164566994, -8.22813017e-05},  //
      {-0.85738802, -8.22814036e-05},   //
      {-1.01091003, -8.22814036e-05},   //
      {-1.01091003, -8.22814036e-05},   //
  });
  TestPoly(polys, 5);
}

TEST(Polygon, Sliver2) {
  Polygons polys;
  polys.push_back({
      {27.4996014, 8.68737034},    //
      {28.27701, 9.528873446},     //
      {27.6687469, 10.881158804},  //
      {27.5080414, 8.7968292242},  //
      {27.5594807, 8.7521896407},  //
      {27.4996014, 8.687370368},   //
  });
  TestPoly(polys, 4);
}

TEST(Polygon, Sliver3) {
  Polygons polys;
  polys.push_back({
      {0, -2.6516838169},               //
      {0, -0.792692184889},             //
      {0, -0.792692184330},             //
      {0, -1.04356134430},              //
      {-0.953957975, -0.768045247331},  //
      {-1.36363637, -0.757460594892},   //
  });
  TestPoly(polys, 4);
}

TEST(Polygon, Sliver4) {
  Polygons polys;
  polys.push_back({
      {-0.375669807, 8.90489388474},  //
      {0, 8.39722729459},             //
      {0, 8.9723053468},              //
      {0, 8.9723053469},              //
      {0, 8.96719646467},             //
      {0, 8.89326191466},             //
      {0, 8.78047276465},             //
      {-0.330551624, 8.8897438473},   //
  });
  TestPoly(polys, 6);
}

TEST(Polygon, Sliver5) {
  Polygons polys;
  polys.push_back({
      {-60, 0},               //
      {-50, 0},               //
      {-50, 0},               //
      {-60, 4.37113897e-07},  //
      {-60, 4.37113897e-07},  //
  });
  polys.push_back({
      {-60, 100},             //
      {-60, 4.37113897e-07},  //
      {-60, 4.37113897e-07},  //
      {-50, 0},               //
  });
  TestPoly(polys, 5);
}

TEST(Polygon, Sliver6) {
  Polygons polys;
  polys.push_back({
      {10, 0},               //
      {0, 10},               //
      {-10, 0},              //
      {-10, 0},              //
      {4.37113897e-07, 10},  //
      {10, 0},               //
  });
  TestPoly(polys, 4);
}

TEST(Polygon, Sliver7) {
  Polygons polys;
  polys.push_back({
      {50, -10},             //
      {60, 0},               //
      {50, 0},               //
      {60, 4.37113897e-07},  //
      {60, 4.37113897e-07},  //
      {60, 4.37113897e-07},  //
      {60, 4.37113897e-07},  //
      {50, 0},               //
  });
  TestPoly(polys, 6);
}

TEST(Polygon, Sliver8) {
  Polygons polys;
  polys.push_back({
      {27.9279995, 4.9749999},    //
      {31.0610008, 2.32299995},   //
      {31.0610008, 2.322999958},  //
      {27.9279995, 4.97499996},   //
      {27.9279995, 4.97499995},   //
      {31.0610008, 2.322999958},  //
      {30.4400005, 5.341000087},  //
  });
  TestPoly(polys, 5);
}

TEST(Polygon, Sliver9) {
  Polygons polys;
  polys.push_back({
      {1.49183154, -0.4795963766194},  //
      {1.71669781, -0.2464189836195},  //
      {1.70233643, -0.2464101176196},  //
      {1.47253978, -0.2462775866197},  //
      {1.47254002, -0.2462870037997},  //
      {1.4869014, -0.2462958697996},   //
  });
  TestPoly(polys, 4);
}

TEST(Polygon, Sliver10) {
  Polygons polys;
  polys.push_back({
      {-1.20783484, 9.82835007},  //
      {-1.12029445, 10.1291866},  //
      {-1.39725339, 10.0962286},  //
      {-1.20783472, 9.82835007},  //
      {-1.39725339, 10.0962286},  //
  });
  TestPoly(polys, 3, 0.000230642);
}

TEST(Polygon, Colinear2) {
  Polygons polys;
  polys.push_back({
      {11.7864399, -7.4572401176},    //
      {11.6818037, -7.309823044873},  //
      {11.6777582, -7.306262028498},  //
      {11.6789398, -7.305788044872},  //
      {11.3459997, -6.83671999889},   //
      {11.25597, -6.9267602888},      //
  });
  TestPoly(polys, 4);
}

TEST(Polygon, Colinear3) {
  Polygons polys;
  polys.push_back({
      {1, -0.600000024},    //
      {1, 0.600000024},     //
      {0, 0.600000024},     //
      {0.5, 0},             //
      {0.5, 0.5},           //
      {0.5, 0.5},           //
      {1, 0.5},             //
      {1, -0.5},            //
      {1, -0.5},            //
      {1, -0.5},            //
      {0.916666627, -0.5},  //
  });
  TestPoly(polys, 9);
}

TEST(Polygon, Envelope) {
  Polygons polys;
  polys.push_back({
      {-36.324749, 0.477457494},  //
      {-36.324749, 0.477457494},  //
      {-36.8497353, 1.03053689},  //
      {-32.5569077, 5},           //
      {-36.8497353, 1.03053689},  //
  });
  polys.push_back({
      {-32.5569077, 5},           //
      {-36.324749, 0.477457494},  //
      {-32.5569077, 5},           //
      {-32.5569077, 5},           //
      {-36.8497353, 1.03053689},  //
  });
  TestPoly(polys, 6);
}

TEST(Polygon, Split) {
  Polygons polys;
  polys.push_back({
      {-0.707106769, -0.707106769},     //
      {1, 0},                           //
      {0.683012664, 0},                 //
      {0.379409522, -0.2329629063},     //
      {0.379409522, -0.2329629062},     //
      {1.49011612e-08, -0.1830126641},  //
      {1.49011612e-08, -0.1830126640},  //
      {-0.14043057, 0},                 //
      {-1, 0},                          //
  });
  TestPoly(polys, 7);
}

TEST(Polygon, Duplicates) {
  Polygons polys;
  polys.push_back({
      {-15, -8.10255623648},         //
      {-15, -9.02439785650},         //
      {-13.636364, -9.4640789678},   //
      {-14.996314, -8.10623646916},  //
      {-15, -8.10639845},            //
      {-15, -8.10255623922},         //
  });
  TestPoly(polys, 4);
}

TEST(Polygon, Simple1) {
  Polygons polys;
  polys.push_back({
      {4.04059982, -4.01843977872},   //
      {3.95867562, -4.252639774604},  //
      {4.23459578, -4.301384938274},  //
      {4.235569, -4.301272878273},    //
      {4.23782539, -4.301418784602},  //
  });
  TestPoly(polys, 3);
}

TEST(Polygon, Simple2) {
  Polygons polys;
  polys.push_back({
      {-1, -1},      //
      {-0.5, -0.5},  //
      {-1, 0},       //
      {0, 1},        //
      {0.5, 0.50},   //
      {1, 1},        //
      {-1, 1},       //
  });
  TestPoly(polys, 5);
}

TEST(Polygon, Simple3) {
  Polygons polys;
  polys.push_back({
      {19.7193489, 6.154459959798},  //
      {20.2308197, 5.642990591187},  //
      {20.3464642, 5.654597767273},  //
      {20.3733711, 5.654040817274},  //
      {20.373394, 5.654040341188},   //
      {20.8738098, 6.154459959801},  //
  });
  TestPoly(polys, 4);
}

TEST(Polygon, Simple4) {
  Polygons polys;
  polys.push_back({
      {15, -12.713556387},          //
      {15, -10.684373988},          //
      {15, -10.684373992},          //
      {15, -11.004141813},          //
      {15, -11.455074309},          //
      {15, -11.455074311},          //
      {14.9958763, -11.454532608},  //
      {14.4307623, -11.380221412},  //
      {13.9298496, -11.276861280},  //
  });
  TestPoly(polys, 7);
}

TEST(Polygon, Simple5) {
  Polygons polys;
  polys.push_back({
      {-27.3845081, 0.37566974864},  //
      {-27.6389656, 0},              //
      {-27.1156006, 0},              //
      {-27.1156006, 0},              //
      {-27.1202412, 0},              //
      {-27.1875362, 0},              //
      {-27.290184, 0},               //
      {-27.3733444, 0.33045187663},  //
  });
  TestPoly(polys, 6);
}

TEST(Polygon, Simple6) {
  Polygons polys;
  polys.push_back({
      {-7.99813318, 12.88888265009},  //
      {-7.85714436, 12.91251955006},  //
      {-7.85714436, 12.98071965005},  //
      {-7.88929749, 12.95930395007},  //
      {-7.99812126, 12.88889125008},  //
  });
  TestPoly(polys, 3);
}

TEST(Polygon, TouchingHole) {
  Polygons polys;
  polys.push_back({
      {-2, -1},  //
      {2, -1},   //
      {2, 1},    //
      {-2, 1},   //
  });
  polys.push_back({
      {-1, -1},  //
      {-1, 1},   //
      {1, 1},    //
      {1, -1},   //
  });
  TestPoly(polys, 8);
}

TEST(Polygon, TouchingHole2) {
  Polygons polys;
  polys.push_back({
      {-0.314814836, -0.462962955},  //
      {-0.240740761, -0.462962955},  //
      {-0.314814836, -0.42592594},   //
  });
  polys.push_back({
      {-0.302469134, -0.450617284},  //
      {-0.302469134, -0.439186096},  //
      {-0.302469134, -0.438271612},  //
      {-0.291037947, -0.438271612},  //
      {-0.290123463, -0.438271612},  //
      {-0.290123463, -0.449702799},  //
      {-0.290123463, -0.450617284},  //
      {-0.30155465, -0.450617284},   //
  });
  TestPoly(polys, 11);
}

TEST(Polygon, Degenerate) {
  Polygons polys;
  polys.push_back({
      {1, -1},   //
      {1, 1},    //
      {1, 1},    //
      {1, -1},   //
      {1, -1},   //
      {-1, -1},  //
      {-1, -1},  //
  });
  TestPoly(polys, 5);
}

TEST(Polygon, Degenerate2) {
  Polygons polys;
  polys.push_back({
      {0.0740740597, -0.314814836829},  //
      {0.0925925896, -0.314814806828},  //
      {0.0925925896, -0.314814806826},  //
      {0.0740740597, -0.314814836830},  //
  });
  TestPoly(polys, 2);
}

TEST(Polygon, Degenerate3) {
  Polygons polys;
  polys.push_back({
      {-0.413580239, -0.2160494039696},  //
      {-0.42592591, -0.2160494039690},   //
      {-0.413580239, -0.2160494039694},  //
      {-0.401234567, -0.2160494039713},  //
      {-0.401234567, -0.2160494039715},  //
      {-0.401234567, -0.2160494039716},  //
      {-0.413580239, -0.2160494039697},  //
  });
  TestPoly(polys, 5);
}

TEST(Polygon, Degenerate4) {
  Polygons polys;
  polys.push_back({
      {0, 10},                        //
      {-0.0696326792, 9.9939079360},  //
      {4.37113897e-07, 10},           //
      {0.636729717, 9.9442930240},    //
  });
  TestPoly(polys, 2);
}

TEST(Polygon, Degenerate5) {
  Polygons polys;
  polys.push_back({
      {1, 0},  //
      {1, 1},  //
      {1, 1},  //
      {1, 1},  //
      {1, 1},  //
      {0, 1},  //
      {0, 1},  //
      {0, 1},  //
  });
  TestPoly(polys, 6);
}

TEST(Polygon, Degenerate6) {
  Polygons polys;
  polys.push_back({
      {4.37113897e-07, -4.37113897e-07},  //
      {4.37113897e-07, 0},                //
      {0, 0},                             //
      {-1.19421679e-06, 0},               //
      {-8.66025352, 0},                   //
      {-8.66025352, 1.3397469},           //
      {-10, -4.37113897e-07},             //
  });
  TestPoly(polys, 5);
}

TEST(Polygon, Tricky) {
  Polygons polys;
  polys.push_back({
      {1, 0},  //
      {2, 1},  //
      {3, 0},  //
      {3, 5},  //
      {2, 5},  //
      {3, 4},  //
      {3, 2},  //
      {3, 3},  //
      {0, 6},  //
  });
  TestPoly(polys, 7);
}

TEST(Polygon, Tricky2) {
  Polygons polys;
  polys.push_back({
      {1, 0},    //
      {3, 1},    //
      {3, 3.5},  //
      {3, 0},    //
      {3, 5},    //
      {2, 5},    //
      {3, 4},    //
      {3, 2},    //
      {3, 3},    //
      {0, 6},    //
  });
  TestPoly(polys, 8);
}

TEST(Polygon, Slit) {
  Polygons polys;
  polys.push_back({
      {27.7069321, 13.514409186},  //
      {28.664566, 10.810289467},   //
      {28.7565536, 10.818337466},  //
      {25.9535275, 19.445167570},  //
      {26.0820198, 18.928167365},  //
      {26.0820198, 18.928167364},  //
      {26.0820198, 18.928167366},  //
      {25.8192234, 18.844831567},  //
      {27.7069321, 13.514409185},  //
      {27.9789181, 13.211655684},  //
  });
  polys.push_back({
      {25.6960907, 20.237478391},  //
      {25.6563644, 20.359741292},  //
      {25.6467285, 20.361473193},  //
  });
  TestPoly(polys, 9);
}

TEST(Polygon, SharedEdge) {
  Polygons polys;
  polys.push_back({
      {0.265432119, 0.00617283491185},   //
      {0.277777791, -3.7252903e-09},     //
      {0.277777791, 0.01851851871184},   //
      {0.240740761, 0.01851851876345},   //
      {0.265432119, 0.006172839551186},  //
      {0.265432119, 0.006172839551187},  //
  });
  TestPoly(polys, 4);
}

TEST(Polygon, Precision) {
  Polygons polys;
  polys.push_back({
      {-0.98486793, -0.492948532},   //
      {-0.984859049, -0.492013603},  //
      {-0.984966695, -0.489926398},  //
      {-0.984955609, -0.490281343},  //
      {-0.985008538, -0.489676297},  //
      {-0.98491329, -0.491925418},   //
      {-0.984878719, -0.492937535},  //
  });
  TestPoly(polys, 5, 0.0001);
};

TEST(Polygon, Precision2) {
  Polygons polys;
  polys.push_back({
      {4.98093176, -0.2479388121113},   //
      {4.94630527, -0.08263990282736},  //
      {4.98092985, -0.2479388122735},   //
  });
  polys.push_back({
      {4.76215458, -0.2478484367566},  //
      {4.76215267, -0.2478604172640},  //
      {4.76215553, -0.2478604172639},  //
  });
  polys.push_back({
      {4.95041943, -0.2417418967815},  //
      {4.85906506, -0.2231211817816},  //
      {4.90268326, -0.1528855567824},  //
      {4.82208872, -0.185909457823},   //
      {4.79133606, -0.2478708772638},  //
      {4.98092985, -0.2479388122733},  //
      {4.90268326, -0.1528855567822},  //
      {4.95041943, -0.2417418967819},  //
  });
  TestPoly(polys, 8);
};

TEST(Polygon, Precision3) {
  Polygons polys;
  polys.push_back({
      {0.630340576, 1.6097908},   //
      {0.777489543, 1.58760202},  //
      {0.890084028, 1.60028839},  //
      {1.05917394, 1.65945554},   //
      {1.06027794, 1.65984178},   //
  });
  TestPoly(polys, 3, 0.00068);
};

TEST(Polygon, DegenerateRing) {
  Polygons polys;
  polys.push_back({
      {-9.82835007, -8.06539154},  //
      {-10.1291866, -7.88505363},  //
      {-10.1291866, -8.2457304},   //
  });
  polys.push_back({
      {-9.82835007, -8.06539154},  //
      {-9.82835007, -8.06539154},  //
      {-9.82835007, -8.06539154},  //
      {-10.1291866, -8.24572945},  //
      {-10.1291866, -7.88505363},  //
      {-9.82835007, -8.06539154},  //
  });
  TestPoly(polys, 9);
}

TEST(Polygon, Sweep) {
  Polygons polys;
  polys.push_back({
      {0, 0.637057483},          //
      {0, 0.637057483},          //
      {1.72745752, 4.72425699},  //
      {1.72745752, 4.72425699},  //
      {1.72745752, 4.72425699},  //
      {1.7274611, 4.72425747},   //
      {0, 0.637057483},          //
      {2.5, 4.82678747},         //
      {1.72745752, 4.72425747},  //
      {0, 0.637057483},          //
      {0, 0.637057483},          //
      {0, 0.637057483},          //
      {0, 0.637057483},          //
      {0, 0.637057483},          //
  });
  TestPoly(polys, 12);
}

TEST(Polygon, Hole) {
  Polygons polys;
  polys.push_back({
      {0.5, -0.5},    //
      {0.5, 0.5},     //
      {-0.5, 0.5},    //
      {-0.5, 0.5},    //
      {-0.5, 0.5},    //
      {-0, 0.5},      //
      {-0.25, 0.25},  //
      {0.25, -0.25},  //
      {0.5, 0},       //
      {0.5, 0},       //
      {0.5, -0.5},    //
      {0.5, -0.5},    //
      {0.5, -0.5},    //
  });
  polys.push_back({
      {0.5, 0},    //
      {-0, 0.5},   //
      {-0, 0.5},   //
      {0.5, 0.5},  //
      {0.5, 0.5},  //
      {0.5, 0.5},  //
  });
  TestPoly(polys, 19);
}

TEST(Polygon, Hole2) {
  Polygons polys;
  polys.push_back({
      {0, -21.8057232},             //
      {0.142438874, -21.3274956},   //
      {0.118176684, -21.1638756},   //
      {2.24720502, -20.7749138},    //
      {0.0947285369, -20.7749138},  //
      {0.0428545363, -19.7292175},  //
  });
  polys.push_back({
      {0.094730109, -20.7749138},  //
      {1.17382383, -20.7749138},   //
      {0.118176684, -21.1638756},  //
  });
  TestPoly(polys, 9);
}

TEST(Polygon, Hole3) {
  Polygons polys;
  polys.push_back({
      {9.42795753, 9.36543179},  //
      {9.41050148, 9.39931583},  //
      {9.39856625, 9.36543179},  //
  });
  polys.push_back({
      {9.3985672, 9.36543179},   //
      {9.39856625, 9.36543179},  //
      {9.39856625, 9.36543179},  //
      {9.39856625, 9.36543274},  //
      {9.41050148, 9.39931488},  //
      {9.41945648, 9.38193321},  //
      {9.41326141, 9.36543179},  //
  });
  TestPoly(polys, 10);
}

TEST(Polygon, Separate3) {
  Polygons polys;
  polys.push_back({
      {-3.46249032, 0},           //
      {-3.46249056, 4.95030928},  //
      {-3.93332648, 1.35438299},  //
  });
  polys.push_back({
      {-5.15814781, 4.8776412},   //
      {-5.11540651, 4.75469398},  //
      {-5.11540651, 4.87985134},  //
      {-5.11540651, 4.87985849},  //
  });
  polys.push_back({
      {-4.29695415, 4.92231846},  //
      {-4.47298813, 2.90674615},  //
      {-4.37842369, 2.634727},    //
      {-4.29695415, 4.92231846},  //
      {-4.29695415, 4.92231846},  //
      {-5.03241968, 4.88416386},  //
      {-5.03241968, 4.51597786},  //
      {-5.03241968, 4.51597786},  //
  });
  TestPoly(polys, 9);
}

TEST(Polygon, Small) {
  Polygons polys;
  polys.push_back({
      {-0.487163663, 0.00357927009},  //
      {-0.49125129, 0.244737789},     //
      {-0.491251856, 0.24427411},     //
      {-0.491260529, 0.245025411},    //
      {-0.49067378, 6.32605588e-05},  //
  });
  TestPoly(polys, 3, 0.0001);
};

TEST(Polygon, Comb) {
  Polygons polys;
  polys.push_back({
      {0.462962955, -0.4272976528},      //
      {0.5, -0.425925917},               //
      {-0.5, -0.42592591283},            //
      {-0.5, -0.462962955269},           //
      {-0.462962985, -0.461591214268},   //
      {-0.462962985, -0.42592591282},    //
      {-0.42592594, -0.42592591280},     //
      {-0.42592594, -0.460219473266},    //
      {-0.351851881, -0.45747599247},    //
      {-0.351851881, -0.42592591257},    //
      {-0.314814836, -0.42592591256},    //
      {-0.314814836, -0.456104249245},   //
      {-0.240740746, -0.453360766132},   //
      {-0.240740746, -0.42592591143},    //
      {-0.203703716, -0.42592591142},    //
      {-0.203703716, -0.451989025130},   //
      {-0.129629642, -0.449245542128},   //
      {-0.129629642, -0.42592591141},    //
      {-0.092592597, -0.42592591119},    //
      {-0.092592597, -0.447873801111},   //
      {-0.0185185187, -0.445130289109},  //
      {-0.0185185187, -0.42592591117},   //
      {0.0185185187, -0.4259259169},     //
      {0.0185185187, -0.44375854762},    //
      {0.0925925896, -0.44101506560},    //
      {0.0925925896, -0.4259259168},     //
      {0.129629627, -0.4259259152},      //
      {0.129629627, -0.43964332341},     //
      {0.203703731, -0.43689984150},     //
      {0.203703731, -0.4259259151},      //
      {0.240740761, -0.4259259149},      //
      {0.240740761, -0.435528148},       //
      {0.314814806, -0.4327846170},      //
      {0.314814806, -0.425925911},       //
      {0.351851851, -0.425925919},       //
      {0.351851851, -0.4314128768},      //
      {0.42592591, -0.4286693931},       //
      {0.42592591, -0.425925912},        //
      {0.462962955, -0.425925919},       //
  });
  TestPoly(polys, 37);
}

TEST(Polygon, Comb2) {
  Polygons polys;
  polys.push_back({
      {-0.5, -0.462962955},             //
      {-0.462962955, -0.461591214},     //
      {-0.462962955, -0.425925913},     //
      {-0.42592591, -0.425925915},      //
      {-0.42592591, -0.460219473},      //
      {-0.351851851, -0.457475998},     //
      {-0.351851851, -0.425925915},     //
      {-0.314814806, -0.425925916},     //
      {-0.314814806, -0.4561042490},    //
      {-0.240740761, -0.45336076632},   //
      {-0.240740761, -0.4259259144},    //
      {-0.203703731, -0.4259259145},    //
      {-0.203703731, -0.45198902534},   //
      {-0.129629627, -0.44924554237},   //
      {-0.129629627, -0.4259259147},    //
      {-0.0925925896, -0.4259259164},   //
      {-0.0925925896, -0.44787380155},  //
      {-0.0185185187, -0.44513028958},  //
      {-0.0185185187, -0.4259259166},   //
      {0.0185185187, -0.42592591113},   //
      {0.0185185187, -0.443758547104},  //
      {0.092592597, -0.441015065107},   //
      {0.092592597, -0.42592591116},    //
      {0.129629642, -0.42592591134},    //
      {0.129629642, -0.439643323122},   //
      {0.203703716, -0.436899841137},   //
      {0.203703716, -0.42592591138},    //
      {0.240740746, -0.42592591140},    //
      {0.240740746, -0.4355281139},     //
      {0.314814836, -0.432784617251},   //
      {0.314814836, -0.42592591252},    //
      {0.351851881, -0.42592591254},    //
      {0.351851881, -0.431412876253},   //
      {0.42592594, -0.428669393272},    //
      {0.42592594, -0.42592591273},     //
      {0.462962985, -0.42592591278},    //
      {0.462962985, -0.427297652277},   //
      {0.5, -0.42592591279},            //
      {-0.5, -0.425925912},             //
  });
  TestPoly(polys, 37);
}

TEST(Polygon, PointPoly) {
  Polygons polys;
  polys.push_back({
      {0, -15.97809792},            //
      {5.08144999, -14.267872844},  //
      {4.83870935, -14.078962343},  //
      {4.38336992, -13.749200837},  //
      {4.35483837, -13.728474638},  //
      {4.33122683, -13.71277935},   //
      {3.87096763, -13.368933730},  //
      {3.52637458, -13.13335511},   //
      {3.38709664, -13.02511889},   //
      {3.10755324, -12.82632165},   //
      {2.90322566, -12.68068413},   //
      {2.80962205, -12.62084011},   //
      {2.41935468, -12.32800489},   //
      {2.16151524, -12.15446578},   //
      {1.93548381, -11.97346316},   //
      {1.56781006, -11.70936397},   //
      {1.45161283, -11.60849956},   //
      {1.02412188, -11.27564533},   //
      {0.967741907, -11.22161294},  //
      {0.628127813, -10.92966180},  //
      {0, -10.92966183},            //
  });
  polys.push_back({
      {15, -10.9296618052},  //
      {15, -10.9296618051},  //
      {15, -10.9296618053},  //
  });
  TestPoly(polys, 20);
};

TEST(Polygon, KissingZigzag) {
  Polygons polys;
  polys.push_back({
      {37.4667358, 0},              //
      {27.8904877, -3.04520559},    //
      {27.8904877, 3.04520559},     //
      {37.4667358, 0},              //
      {36.4568138, 8.64042377},     //
      {27.8409691, 3.46886754},     //
      {26.4364243, 9.39511108},     //
      {36.4568138, 8.64042377},     //
      {33.4814987, 16.8150406},     //
      {26.2905369, 9.79593277},     //
      {23.5571651, 15.23852350},    //
      {33.4814987, 16.81504061},    //
      {28.7011852, 24.08315282},    //
      {23.322773, 15.59489543},     //
      {19.4079323, 20.26042184},    //
      {28.7011852, 24.08315285},    //
      {22.3735847, 30.05293856},    //
      {19.0976753, 20.55313687},    //
      {14.2124147, 24.19007688},    //
      {22.3735847, 30.05293859},    //
      {14.8398142, 34.40256120},    //
      {13.8430138, 24.40335081},    //
      {8.2507, 26.81563952},        //
      {14.8398142, 34.40256123},    //
      {6.50603199, 36.89752964},    //
      {7.84207535, 26.9379735},     //
      {1.84419155, 27.99556356},    //
      {6.50603199, 36.89752967},    //
      {-2.1784966, 37.40335088},    //
      {1.41836619, 28.02036489},    //
      {-4.66174126, 27.66623880},   //
      {-2.1784966, 37.40335081},    //
      {-10.7455816, 35.89273832},   //
      {-5.08180761, 27.59216883},   //
      {-10.916357, 25.84541894},    //
      {-10.7455816, 35.89273835},   //
      {-18.7333698, 32.44714366},   //
      {-11.3080206, 25.67647177},   //
      {-16.5824718, 22.63126758},   //
      {-18.7333698, 32.44714369},   //
      {-25.711235, 27.25231360},    //
      {-16.924614, 22.37655071},    //
      {-21.3546181, 18.19705772},   //
      {-25.711235, 27.25231363},    //
      {-31.3030052, 20.58830454},   //
      {-21.6287994, 17.87030415},   //
      {-24.9755325, 12.78183846},   //
      {-31.3030052, 20.58830457},   //
      {-35.2072144, 12.81438068},   //
      {-25.1669636, 12.40066729},   //
      {-27.2500057, 6.677553180},   //
      {-35.2072144, 12.81438061},   //
      {-37.213398, 4.349624632},    //
      {-27.3483734, 6.262504583},   //
      {-28.0554276, 0.2132747174},  //
      {-37.213398, 4.349624635},    //
      {-37.213398, -4.349621776},   //
      {-28.0554276, -0.213272817},  //
      {-27.3483734, -6.262502678},  //
      {-37.213398, -4.349621779},   //
      {-35.2072144, -12.81437870},  //
      {-27.2500057, -6.677552221},  //
      {-25.1669636, -12.40066622},  //
      {-35.2072144, -12.81437873},  //
      {-31.3029995, -20.58831024},  //
      {-24.9755306, -12.78184325},  //
      {-21.6287937, -17.87030796},  //
      {-31.3029995, -20.58831027},  //
      {-25.7112312, -27.25231938},  //
      {-21.3546143, -18.19706159},  //
      {-16.9246101, -22.37655450},  //
      {-25.7112312, -27.25231931},  //
      {-18.7333641, -32.44714742},  //
      {-16.5824661, -22.63126953},  //
      {-11.3080158, -25.67647364},  //
      {-18.7333641, -32.44714745},  //
      {-10.7455835, -35.89273836},  //
      {-10.9163589, -25.84541897},  //
      {-5.08180904, -27.59216888},  //
      {-10.7455835, -35.89273839},  //
      {-2.17849016, -37.40335080},  //
      {-4.66173601, -27.66623881},  //
      {1.41837108, -28.02036482},   //
      {-2.17849016, -37.40335083},  //
      {6.50602913, -36.89752964},   //
      {1.84418964, -27.99556355},   //
      {7.84207344, -26.9379736},    //
      {6.50602913, -36.89752967},   //
      {14.8398247, -34.40255748},   //
      {8.25070763, -26.81563579},   //
      {13.8430195, -24.4033470},    //
      {14.8398247, -34.40255741},   //
      {22.3735847, -30.05293852},   //
      {14.2124147, -24.19007683},   //
      {19.0976753, -20.55313684},   //
      {22.3735847, -30.05293855},   //
      {28.7011795, -24.08315856},   //
      {19.4079285, -20.26042567},   //
      {23.3227692, -15.59490118},   //
      {28.7011795, -24.08315859},   //
      {33.4815025, -16.815036800},  //
      {23.5571671, -15.238520601},  //
      {26.2905388, -9.7959299102},  //
      {33.4815025, -16.815036803},  //
      {36.4568138, -8.6404275904},  //
      {26.4364243, -9.3951129905},  //
      {27.8409691, -3.4688699206},  //
      {36.4568138, -8.6404275907},  //
  });
  TestPoly(polys, 106);
}

TEST(Polygon, Sponge) {
  Polygons polys;
  polys.push_back({
      {-0.5, -0.5},                   //
      {-0.388888896, -0.3888888963},  //
      {-0.388888896, -0.3888888964},  //
      {-0.388888896, -0.2777777916},  //
      {-0.388888896, -0.2777777917},  //
      {-0.277777791, -0.2777777910},  //
      {-0.166666657, -0.1666666721},  //
      {-0.166666672, -0.1666666722},  //
      {-0.166666672, -0.1666666723},  //
      {-0.166666672, 0.1666666726},   //
      {-0.166666672, 0.1666666727},   //
      {0.166666657, 0.16666667242},   //
      {0.277777791, 0.27777779153},   //
      {0.277777791, 0.27777779154},   //
      {0.277777791, 0.27777779155},   //
      {0.277777791, 0.38888889656},   //
      {0.277777791, 0.38888889657},   //
      {0.388888896, 0.38888889661},   //
      {0.388888896, 0.38888889660},   //
      {0.5, 0.5},                     //
      {-0.5, 0.5},                    //
  });
  polys.push_back({
      {-0.388888896, -0.0555555563},  //
      {-0.388888896, -0.0555555564},  //
      {-0.388888896, 0.0555555563},   //
      {-0.388888896, 0.0555555564},   //
      {-0.277777791, 0.0555555566},   //
      {-0.277777791, 0.0555555565},   //
      {-0.277777791, -0.0555555565},  //
      {-0.277777791, -0.0555555566},  //
  });
  polys.push_back({
      {-0.277777791, 0.2777777912},  //
      {-0.277777791, 0.2777777913},  //
      {-0.388888896, 0.2777777917},  //
      {-0.388888896, 0.2777777918},  //
      {-0.388888896, 0.3888888960},  //
      {-0.388888896, 0.3888888961},  //
      {-0.277777791, 0.3888888964},  //
      {-0.277777791, 0.3888888965},  //
  });
  polys.push_back({
      {-0.055555556, 0.2777777918},  //
      {-0.055555556, 0.2777777919},  //
      {-0.055555556, 0.3888888960},  //
      {-0.055555556, 0.3888888961},  //
      {0.055555556, 0.38888889647},  //
      {0.055555556, 0.38888889648},  //
      {0.055555556, 0.27777779146},  //
      {0.055555556, 0.27777779145},  //
  });
  TestPoly(polys, 49);
}

TEST(Polygon, Sponge2) {
  Polygons polys;
  polys.push_back({
      {0.5, -0.5},                  //
      {0.5, 0.5},                   //
      {-0.5, 0.5},                  //
      {-0.388888896, 0.388888896},  //
      {-0.388888896, 0.388888896},  //
      {-0.277777791, 0.388888896},  //
      {-0.277777791, 0.388888896},  //
      {-0.277777791, 0.388888896},  //
      {-0.277777791, 0.277777791},  //
      {-0.277777791, 0.277777791},  //
      {-0.166666657, 0.166666672},  //
      {-0.166666657, 0.166666672},  //
      {0.166666672, 0.166666672},   //
      {0.166666672, 0.166666672},   //
      {0.166666672, 0.166666672},   //
      {0.166666672, 0.166666672},   //
      {0.166666672, 0.166666672},   //
      {0.166666672, -0.166666672},  //
      {0.166666672, -0.166666672},  //
      {0.166666672, -0.166666672},  //
      {0.166666672, -0.166666672},  //
      {0.166666657, -0.166666672},  //
      {0.277777791, -0.277777791},  //
      {0.388888896, -0.277777791},  //
      {0.388888896, -0.277777791},  //
      {0.388888896, -0.277777791},  //
      {0.388888896, -0.388888896},  //
  });
  polys.push_back({
      {0.388888896, -0.055555556},  //
      {0.277777791, -0.055555556},  //
      {0.277777791, -0.055555556},  //
      {0.277777791, -0.055555556},  //
      {0.277777791, 0.055555556},   //
      {0.388888896, 0.055555556},   //
      {0.388888896, 0.055555556},   //
      {0.388888896, 0.055555556},   //
  });
  polys.push_back({
      {0.388888896, 0.277777791},  //
      {0.277777791, 0.277777791},  //
      {0.277777791, 0.277777791},  //
      {0.277777791, 0.277777791},  //
      {0.277777791, 0.388888896},  //
      {0.388888896, 0.388888896},  //
      {0.388888896, 0.388888896},  //
      {0.388888896, 0.388888896},  //
  });
  polys.push_back({
      {0.055555556, 0.277777791},   //
      {0.055555556, 0.277777791},   //
      {-0.055555556, 0.277777791},  //
      {-0.055555556, 0.277777791},  //
      {-0.055555556, 0.388888896},  //
      {-0.055555556, 0.388888896},  //
      {0.055555556, 0.388888896},   //
      {0.055555556, 0.388888896},   //
  });
  TestPoly(polys, 55);
}

TEST(Polygon, SquareHoles) {
  Polygons polys;
  polys.push_back({
      {0.388888896, -0.2777777910},  //
      {0.388888896, -0.388888896},   //
      {0.277777791, -0.388888896},   //
      {0.277777791, -0.2777777911},  //
  });
  polys.push_back({
      {0.388888896, -0.0555555564},  //
      {0.277777791, -0.0555555562},  //
      {0.277777791, 0.0555555566},   //
      {0.388888896, 0.0555555565},   //
  });
  polys.push_back({
      {0.055555556, -0.2777777915},   //
      {0.055555556, -0.3888888963},   //
      {-0.055555556, -0.3888888963},  //
      {-0.055555556, -0.2777777914},  //
  });
  polys.push_back({
      {0.388888896, 0.2777777918},  //
      {0.277777791, 0.2777777917},  //
      {0.277777791, 0.3888888960},  //
      {0.388888896, 0.3888888969},  //
  });
  polys.push_back({
      {0.166666672, 0.1666666722},    //
      {0.166666672, -0.1666666721},   //
      {-0.166666672, -0.1666666726},  //
      {-0.166666672, 0.1666666723},   //
  });
  polys.push_back({
      {0.055555556, 0.3888888965},   //
      {0.055555556, 0.2777777912},   //
      {-0.055555556, 0.2777777914},  //
      {-0.055555556, 0.3888888966},  //
  });
  polys.push_back({
      {-0.277777791, -0.2777777910},  //
      {-0.277777791, -0.3888888967},  //
      {-0.388888896, -0.3888888968},  //
      {-0.388888896, -0.2777777919},  //
  });
  polys.push_back({
      {-0.277777791, -0.0555555567},  //
      {-0.388888896, -0.0555555561},  //
      {-0.388888896, 0.0555555569},   //
      {-0.277777791, 0.0555555568},   //
  });
  polys.push_back({
      {-0.277777791, 0.3888888962},  //
      {-0.277777791, 0.2777777910},  //
      {-0.388888896, 0.2777777911},  //
      {-0.388888896, 0.3888888963},  //
  });
  polys.push_back({
      {-0.5, -0.5},  //
      {0.5, -0.5},   //
      {0.5, 0.5},    //
      {-0.5, 0.5},   //
  });
  TestPoly(polys, 56);
}

TEST(Polygon, Sponge3) {
  Polygons polys;
  polys.push_back({
      {0.5, -0.166666672},            //
      {0.351851851, -0.117283955},    //
      {0.351851851, -0.129629627},    //
      {0.339506179, -0.129629627},    //
      {0.314814836, -0.129629627},    //
      {0.314814836, -0.117283948},    //
      {0.314814836, -0.104938276},    //
      {0.166666672, -0.0555555597},   //
      {0.166666672, -0.166666672},    //
      {0.0555555634, -0.166666672},   //
      {-0.166666672, -0.166666672},   //
      {-0.166666672, -0.0555555634},  //
      {-0.166666672, 0.0555555597},   //
      {-0.314814836, 0.104938276},    //
      {-0.314814836, 0.092592597},    //
      {-0.327160507, 0.092592597},    //
      {-0.351851851, 0.092592597},    //
      {-0.351851851, 0.104938276},    //
      {-0.351851851, 0.117283955},    //
      {-0.5, 0.166666672},            //
      {-0.5, -0.166666672},           //
  });
  polys.push_back({
      {0.240740761, -0.092592597},  //
      {0.240740761, -0.104938276},  //
      {0.240740761, -0.129629627},  //
      {0.228395075, -0.129629627},  //
      {0.203703716, -0.129629627},  //
      {0.203703716, -0.117283948},  //
      {0.203703716, -0.092592597},  //
      {0.216049403, -0.092592597},  //
  });
  polys.push_back({
      {-0.203703716, -0.129629627},  //
      {-0.216049403, -0.129629627},  //
      {-0.240740761, -0.129629627},  //
      {-0.240740761, -0.117283948},  //
      {-0.240740761, -0.092592597},  //
      {-0.228395075, -0.092592597},  //
      {-0.203703716, -0.092592597},  //
      {-0.203703716, -0.104938276},  //
  });
  polys.push_back({
      {-0.203703716, -0.0185185187},   //
      {-0.216049403, -0.0185185187},   //
      {-0.240740761, -0.0185185187},   //
      {-0.240740761, -0.00617284048},  //
      {-0.240740761, 0.0185185187},    //
      {-0.228395075, 0.0185185187},    //
      {-0.203703716, 0.0185185187},    //
      {-0.203703716, 0.00617284048},   //
  });
  polys.push_back({
      {-0.314814836, -0.129629627},  //
      {-0.327160507, -0.129629627},  //
      {-0.351851851, -0.129629627},  //
      {-0.351851851, -0.117283948},  //
      {-0.351851851, -0.092592597},  //
      {-0.339506179, -0.092592597},  //
      {-0.314814836, -0.092592597},  //
      {-0.314814836, -0.104938276},  //
  });
  polys.push_back({
      {-0.42592594, -0.129629627},   //
      {-0.438271612, -0.129629627},  //
      {-0.462962955, -0.129629627},  //
      {-0.462962955, -0.117283948},  //
      {-0.462962955, -0.092592597},  //
      {-0.450617284, -0.092592597},  //
      {-0.42592594, -0.092592597},   //
      {-0.42592594, -0.104938276},   //
  });
  polys.push_back({
      {-0.277777791, -0.055555556},   //
      {-0.314814836, -0.055555556},   //
      {-0.388888896, -0.055555556},   //
      {-0.388888896, -0.0185185205},  //
      {-0.388888896, 0.055555556},    //
      {-0.351851851, 0.055555556},    //
      {-0.277777791, 0.055555556},    //
      {-0.277777791, 0.0185185205},   //
  });
  polys.push_back({
      {-0.42592594, -0.0185185187},    //
      {-0.438271612, -0.0185185187},   //
      {-0.462962955, -0.0185185187},   //
      {-0.462962955, -0.00617284048},  //
      {-0.462962955, 0.0185185187},    //
      {-0.450617284, 0.0185185187},    //
      {-0.42592594, 0.0185185187},     //
      {-0.42592594, 0.00617284048},    //
  });
  polys.push_back({
      {-0.42592594, 0.092592597},   //
      {-0.438271612, 0.092592597},  //
      {-0.462962955, 0.092592597},  //
      {-0.462962955, 0.104938276},  //
      {-0.462962955, 0.129629627},  //
      {-0.450617284, 0.129629627},  //
      {-0.42592594, 0.129629627},   //
      {-0.42592594, 0.117283948},   //
  });
  TestPoly(polys, 99);
}

TEST(Polygon, SpongeMore) {
  Polygons polys;
  polys.push_back({
      {-0.376543224, 0.0802469105},  //
      {-0.277777791, 0.277777791},   //
      {-0.388888896, 0.277777791},   //
  });
  polys.push_back({
      {-0.364197552, 0.104938269},  //
      {-0.376543224, 0.117283955},  //
      {-0.364197552, 0.117283955},  //
  });
  polys.push_back({
      {-0.364197552, 0.141975313},  //
      {-0.376543224, 0.154320985},  //
      {-0.364197552, 0.154320985},  //
  });
  polys.push_back({
      {-0.339506179, 0.19135803},   //
      {-0.327160507, 0.19135803},   //
      {-0.327160507, 0.179012358},  //
  });
  polys.push_back({
      {-0.314814836, 0.203703716},  //
      {-0.351851851, 0.240740761},  //
      {-0.314814836, 0.240740761},  //
  });
  TestPoly(polys, 21);
}

TEST(Polygon, Sponge3a) {
  Polygons polys;
  polys.push_back({
      {0.5, -0.5},                   //
      {0.462962955, -0.462962955},   //
      {0.462962955, -0.462962955},   //
      {0.42592594, -0.462962955},    //
      {0.42592594, -0.462962955},    //
      {0.42592594, -0.462962955},    //
      {0.42592594, -0.42592594},     //
      {0.42592594, -0.42592594},     //
      {0.388888896, -0.388888896},   //
      {0.388888896, -0.388888896},   //
      {0.388888896, -0.388888896},   //
      {0.388888896, -0.388888896},   //
      {0.277777791, -0.388888896},   //
      {0.277777791, -0.388888896},   //
      {0.277777791, -0.388888896},   //
      {0.277777791, -0.388888896},   //
      {0.277777791, -0.388888896},   //
      {0.277777791, -0.388888896},   //
      {0.277777791, -0.277777791},   //
      {0.277777791, -0.277777791},   //
      {0.277777791, -0.277777791},   //
      {0.277777791, -0.277777791},   //
      {0.240740746, -0.240740761},   //
      {0.203703716, -0.240740761},   //
      {0.203703716, -0.240740761},   //
      {0.203703716, -0.240740761},   //
      {0.203703716, -0.203703716},   //
      {0.203703731, -0.203703716},   //
      {0.166666657, -0.166666672},   //
      {0.166666657, -0.166666672},   //
      {0.166666657, -0.166666672},   //
      {0.166666657, -0.166666672},   //
      {0.166666657, -0.166666672},   //
      {0.166666657, -0.166666672},   //
      {-0.166666672, -0.166666672},  //
      {-0.166666672, -0.166666672},  //
      {-0.166666672, -0.166666672},  //
      {-0.166666672, -0.166666672},  //
      {-0.166666672, -0.166666672},  //
      {-0.166666672, -0.166666672},  //
      {-0.166666672, -0.166666672},  //
      {-0.166666672, -0.166666672},  //
      {-0.166666672, -0.166666672},  //
      {-0.166666672, -0.166666672},  //
      {-0.166666672, -0.166666672},  //
      {-0.166666672, -0.166666672},  //
      {-0.166666672, -0.166666672},  //
      {-0.166666672, -0.166666672},  //
      {-0.166666672, 0.166666672},   //
      {-0.166666672, 0.166666672},   //
      {-0.166666672, 0.166666672},   //
      {-0.166666672, 0.166666672},   //
      {-0.166666672, 0.166666672},   //
      {-0.166666672, 0.166666672},   //
      {-0.166666672, 0.166666672},   //
      {-0.166666672, 0.166666672},   //
      {-0.166666672, 0.166666672},   //
      {-0.166666672, 0.166666672},   //
      {-0.166666672, 0.166666672},   //
      {-0.166666672, 0.166666672},   //
      {-0.166666657, 0.166666672},   //
      {-0.203703731, 0.203703716},   //
      {-0.240740761, 0.203703716},   //
      {-0.240740761, 0.203703716},   //
      {-0.240740761, 0.203703716},   //
      {-0.240740761, 0.240740761},   //
      {-0.240740746, 0.240740761},   //
      {-0.277777791, 0.277777791},   //
      {-0.277777791, 0.277777791},   //
      {-0.277777791, 0.277777791},   //
      {-0.388888896, 0.277777791},   //
      {-0.388888896, 0.277777791},   //
      {-0.388888896, 0.277777791},   //
      {-0.388888896, 0.277777791},   //
      {-0.388888896, 0.277777791},   //
      {-0.388888896, 0.277777791},   //
      {-0.388888896, 0.388888896},   //
      {-0.388888896, 0.388888896},   //
      {-0.42592594, 0.42592594},     //
      {-0.462962955, 0.42592594},    //
      {-0.462962955, 0.42592594},    //
      {-0.462962955, 0.42592594},    //
      {-0.462962955, 0.462962955},   //
      {-0.5, 0.5},                   //
      {-0.5, -0.5},                  //
  });
  polys.push_back({
      {0.351851851, -0.462962955},  //
      {0.351851851, -0.462962955},  //
      {0.314814836, -0.462962955},  //
      {0.314814836, -0.462962955},  //
      {0.314814836, -0.42592594},   //
      {0.314814836, -0.42592594},   //
      {0.351851851, -0.42592594},   //
      {0.351851851, -0.42592594},   //
  });
  polys.push_back({
      {0.240740761, -0.462962955},  //
      {0.203703716, -0.462962955},  //
      {0.203703716, -0.462962955},  //
      {0.203703716, -0.462962955},  //
      {0.203703716, -0.42592594},   //
      {0.240740761, -0.42592594},   //
      {0.240740761, -0.42592594},   //
      {0.240740761, -0.42592594},   //
  });
  polys.push_back({
      {0.129629627, -0.462962955},  //
      {0.129629627, -0.462962955},  //
      {0.092592597, -0.462962955},  //
      {0.092592597, -0.462962955},  //
      {0.092592597, -0.42592594},   //
      {0.092592597, -0.42592594},   //
      {0.129629627, -0.42592594},   //
      {0.129629627, -0.42592594},   //
  });
  polys.push_back({
      {0.240740761, -0.351851851},  //
      {0.203703716, -0.351851851},  //
      {0.203703716, -0.351851851},  //
      {0.203703716, -0.351851851},  //
      {0.203703716, -0.314814836},  //
      {0.240740761, -0.314814836},  //
      {0.240740761, -0.314814836},  //
      {0.240740761, -0.314814836},  //
  });
  polys.push_back({
      {0.129629627, -0.351851851},  //
      {0.092592597, -0.351851851},  //
      {0.092592597, -0.351851851},  //
      {0.092592597, -0.314814836},  //
      {0.092592597, -0.314814836},  //
      {0.129629627, -0.314814836},  //
      {0.129629627, -0.314814836},  //
      {0.129629627, -0.351851851},  //
  });
  polys.push_back({
      {0.0185185187, -0.462962955},   //
      {0.0185185187, -0.462962955},   //
      {-0.0185185187, -0.462962955},  //
      {-0.0185185187, -0.462962955},  //
      {-0.0185185187, -0.42592594},   //
      {-0.0185185187, -0.42592594},   //
      {0.0185185187, -0.42592594},    //
      {0.0185185187, -0.42592594},    //
  });
  polys.push_back({
      {0.055555556, -0.388888896},   //
      {0.055555556, -0.388888896},   //
      {0.055555556, -0.388888896},   //
      {-0.055555556, -0.388888896},  //
      {-0.055555556, -0.388888896},  //
      {-0.055555556, -0.388888896},  //
      {-0.055555556, -0.388888896},  //
      {-0.055555556, -0.388888896},  //
      {-0.055555556, -0.388888896},  //
      {-0.055555556, -0.277777791},  //
      {-0.055555556, -0.277777791},  //
      {-0.055555556, -0.277777791},  //
      {-0.055555556, -0.277777791},  //
      {-0.055555556, -0.277777791},  //
      {-0.055555556, -0.277777791},  //
      {0.055555556, -0.277777791},   //
      {0.055555556, -0.277777791},   //
      {0.055555556, -0.277777791},   //
      {0.055555556, -0.277777791},   //
      {0.055555556, -0.277777791},   //
      {0.055555556, -0.388888896},   //
      {0.055555556, -0.388888896},   //
      {0.055555556, -0.388888896},   //
  });
  polys.push_back({
      {0.129629627, -0.240740761},  //
      {0.092592597, -0.240740761},  //
      {0.092592597, -0.240740761},  //
      {0.092592597, -0.203703716},  //
      {0.092592597, -0.203703716},  //
      {0.129629627, -0.203703716},  //
      {0.129629627, -0.203703716},  //
      {0.129629627, -0.240740761},  //
  });
  polys.push_back({
      {0.0185185187, -0.240740761},   //
      {-0.0185185187, -0.240740761},  //
      {-0.0185185187, -0.240740761},  //
      {-0.0185185187, -0.203703716},  //
      {-0.0185185187, -0.203703716},  //
      {0.0185185187, -0.203703716},   //
      {0.0185185187, -0.203703716},   //
      {0.0185185187, -0.240740761},   //
  });
  polys.push_back({
      {-0.092592597, -0.462962955},  //
      {-0.092592597, -0.462962955},  //
      {-0.129629627, -0.462962955},  //
      {-0.129629627, -0.462962955},  //
      {-0.129629627, -0.42592594},   //
      {-0.129629627, -0.42592594},   //
      {-0.092592597, -0.42592594},   //
      {-0.092592597, -0.42592594},   //
  });
  polys.push_back({
      {-0.092592597, -0.351851851},  //
      {-0.129629627, -0.351851851},  //
      {-0.129629627, -0.351851851},  //
      {-0.129629627, -0.314814836},  //
      {-0.129629627, -0.314814836},  //
      {-0.092592597, -0.314814836},  //
      {-0.092592597, -0.314814836},  //
      {-0.092592597, -0.351851851},  //
  });
  polys.push_back({
      {-0.203703716, -0.462962955},  //
      {-0.240740761, -0.462962955},  //
      {-0.240740761, -0.462962955},  //
      {-0.240740761, -0.462962955},  //
      {-0.240740761, -0.42592594},   //
      {-0.203703716, -0.42592594},   //
      {-0.203703716, -0.42592594},   //
      {-0.203703716, -0.42592594},   //
  });
  polys.push_back({
      {-0.203703716, -0.351851851},  //
      {-0.240740761, -0.351851851},  //
      {-0.240740761, -0.351851851},  //
      {-0.240740761, -0.351851851},  //
      {-0.240740761, -0.314814836},  //
      {-0.203703716, -0.314814836},  //
      {-0.203703716, -0.314814836},  //
      {-0.203703716, -0.314814836},  //
  });
  polys.push_back({
      {-0.092592597, -0.240740761},  //
      {-0.129629627, -0.240740761},  //
      {-0.129629627, -0.240740761},  //
      {-0.129629627, -0.203703716},  //
      {-0.129629627, -0.203703716},  //
      {-0.092592597, -0.203703716},  //
      {-0.092592597, -0.203703716},  //
      {-0.092592597, -0.240740761},  //
  });
  polys.push_back({
      {-0.203703716, -0.240740761},  //
      {-0.240740761, -0.240740761},  //
      {-0.240740761, -0.240740761},  //
      {-0.240740761, -0.240740761},  //
      {-0.240740761, -0.203703716},  //
      {-0.203703716, -0.203703716},  //
      {-0.203703716, -0.203703716},  //
      {-0.203703716, -0.203703716},  //
  });
  polys.push_back({
      {-0.240740761, -0.129629627},  //
      {-0.240740761, -0.129629627},  //
      {-0.240740761, -0.092592597},  //
      {-0.203703716, -0.092592597},  //
      {-0.203703716, -0.092592597},  //
      {-0.203703716, -0.092592597},  //
      {-0.203703716, -0.129629627},  //
      {-0.240740761, -0.129629627},  //
  });
  polys.push_back({
      {-0.203703716, -0.0185185187},  //
      {-0.240740761, -0.0185185187},  //
      {-0.240740761, -0.0185185187},  //
      {-0.240740761, -0.0185185187},  //
      {-0.240740761, 0.0185185187},   //
      {-0.203703716, 0.0185185187},   //
      {-0.203703716, 0.0185185187},   //
      {-0.203703716, 0.0185185187},   //
  });
  polys.push_back({
      {-0.277777791, -0.388888896},  //
      {-0.277777791, -0.388888896},  //
      {-0.277777791, -0.388888896},  //
      {-0.388888896, -0.388888896},  //
      {-0.388888896, -0.388888896},  //
      {-0.388888896, -0.388888896},  //
      {-0.388888896, -0.388888896},  //
      {-0.388888896, -0.388888896},  //
      {-0.388888896, -0.388888896},  //
      {-0.388888896, -0.388888896},  //
      {-0.388888896, -0.277777791},  //
      {-0.388888896, -0.277777791},  //
      {-0.388888896, -0.277777791},  //
      {-0.388888896, -0.277777791},  //
      {-0.277777791, -0.277777791},  //
      {-0.277777791, -0.277777791},  //
      {-0.277777791, -0.277777791},  //
      {-0.277777791, -0.277777791},  //
      {-0.277777791, -0.277777791},  //
      {-0.277777791, -0.388888896},  //
      {-0.277777791, -0.388888896},  //
  });
  polys.push_back({
      {-0.314814836, -0.462962955},  //
      {-0.314814836, -0.462962955},  //
      {-0.351851851, -0.462962955},  //
      {-0.351851851, -0.462962955},  //
      {-0.351851851, -0.42592594},   //
      {-0.351851851, -0.42592594},   //
      {-0.314814836, -0.42592594},   //
      {-0.314814836, -0.42592594},   //
  });
  polys.push_back({
      {-0.42592594, -0.462962955},   //
      {-0.462962955, -0.462962955},  //
      {-0.462962955, -0.462962955},  //
      {-0.462962955, -0.462962955},  //
      {-0.462962955, -0.42592594},   //
      {-0.42592594, -0.42592594},    //
      {-0.42592594, -0.42592594},    //
      {-0.42592594, -0.42592594},    //
  });
  polys.push_back({
      {-0.42592594, -0.351851851},   //
      {-0.462962955, -0.351851851},  //
      {-0.462962955, -0.351851851},  //
      {-0.462962955, -0.351851851},  //
      {-0.462962955, -0.314814836},  //
      {-0.42592594, -0.314814836},   //
      {-0.42592594, -0.314814836},   //
      {-0.42592594, -0.314814836},   //
  });
  polys.push_back({
      {-0.314814836, -0.240740761},  //
      {-0.351851851, -0.240740761},  //
      {-0.351851851, -0.240740761},  //
      {-0.351851851, -0.203703716},  //
      {-0.351851851, -0.203703716},  //
      {-0.314814836, -0.203703716},  //
      {-0.314814836, -0.203703716},  //
      {-0.314814836, -0.240740761},  //
  });
  polys.push_back({
      {-0.351851851, -0.129629627},  //
      {-0.351851851, -0.092592597},  //
      {-0.351851851, -0.092592597},  //
      {-0.314814836, -0.092592597},  //
      {-0.314814836, -0.092592597},  //
      {-0.314814836, -0.129629627},  //
      {-0.314814836, -0.129629627},  //
      {-0.351851851, -0.129629627},  //
  });
  polys.push_back({
      {-0.277777791, -0.055555556},  //
      {-0.277777791, -0.055555556},  //
      {-0.277777791, -0.055555556},  //
      {-0.277777791, -0.055555556},  //
      {-0.388888896, -0.055555556},  //
      {-0.388888896, -0.055555556},  //
      {-0.388888896, -0.055555556},  //
      {-0.388888896, -0.055555556},  //
      {-0.388888896, -0.055555556},  //
      {-0.388888896, -0.055555556},  //
      {-0.388888896, 0.055555556},   //
      {-0.388888896, 0.055555556},   //
      {-0.388888896, 0.055555556},   //
      {-0.388888896, 0.055555556},   //
      {-0.388888896, 0.055555556},   //
      {-0.277777791, 0.055555556},   //
      {-0.277777791, 0.055555556},   //
      {-0.277777791, 0.055555556},   //
      {-0.277777791, 0.055555556},   //
      {-0.277777791, -0.055555556},  //
      {-0.277777791, -0.055555556},  //
  });
  polys.push_back({
      {-0.42592594, -0.240740761},   //
      {-0.462962955, -0.240740761},  //
      {-0.462962955, -0.240740761},  //
      {-0.462962955, -0.240740761},  //
      {-0.462962955, -0.203703716},  //
      {-0.42592594, -0.203703716},   //
      {-0.42592594, -0.203703716},   //
      {-0.42592594, -0.203703716},   //
  });
  polys.push_back({
      {-0.462962955, -0.129629627},  //
      {-0.462962955, -0.129629627},  //
      {-0.462962955, -0.092592597},  //
      {-0.42592594, -0.092592597},   //
      {-0.42592594, -0.092592597},   //
      {-0.42592594, -0.092592597},   //
      {-0.42592594, -0.129629627},   //
      {-0.462962955, -0.129629627},  //
  });
  polys.push_back({
      {-0.42592594, -0.0185185187},   //
      {-0.462962955, -0.0185185187},  //
      {-0.462962955, -0.0185185187},  //
      {-0.462962955, -0.0185185187},  //
      {-0.462962955, 0.0185185187},   //
      {-0.42592594, 0.0185185187},    //
      {-0.42592594, 0.0185185187},    //
      {-0.42592594, 0.0185185187},    //
  });
  polys.push_back({
      {-0.203703716, 0.092592597},  //
      {-0.240740761, 0.092592597},  //
      {-0.240740761, 0.092592597},  //
      {-0.240740761, 0.092592597},  //
      {-0.240740761, 0.129629627},  //
      {-0.203703716, 0.129629627},  //
      {-0.203703716, 0.129629627},  //
      {-0.203703716, 0.129629627},  //
  });
  polys.push_back({
      {-0.351851851, 0.092592597},  //
      {-0.351851851, 0.129629627},  //
      {-0.351851851, 0.129629627},  //
      {-0.314814836, 0.129629627},  //
      {-0.314814836, 0.129629627},  //
      {-0.314814836, 0.092592597},  //
      {-0.314814836, 0.092592597},  //
      {-0.351851851, 0.092592597},  //
  });
  polys.push_back({
      {-0.314814836, 0.203703716},  //
      {-0.351851851, 0.203703716},  //
      {-0.351851851, 0.203703716},  //
      {-0.351851851, 0.240740761},  //
      {-0.351851851, 0.240740761},  //
      {-0.314814836, 0.240740761},  //
      {-0.314814836, 0.240740761},  //
      {-0.314814836, 0.203703716},  //
  });
  polys.push_back({
      {-0.42592594, 0.092592597},   //
      {-0.462962955, 0.092592597},  //
      {-0.462962955, 0.092592597},  //
      {-0.462962955, 0.092592597},  //
      {-0.462962955, 0.129629627},  //
      {-0.42592594, 0.129629627},   //
      {-0.42592594, 0.129629627},   //
      {-0.42592594, 0.129629627},   //
  });
  polys.push_back({
      {-0.42592594, 0.203703716},   //
      {-0.462962955, 0.203703716},  //
      {-0.462962955, 0.203703716},  //
      {-0.462962955, 0.203703716},  //
      {-0.462962955, 0.240740761},  //
      {-0.42592594, 0.240740761},   //
      {-0.42592594, 0.240740761},   //
      {-0.42592594, 0.240740761},   //
  });
  polys.push_back({
      {-0.42592594, 0.314814836},   //
      {-0.462962955, 0.314814836},  //
      {-0.462962955, 0.314814836},  //
      {-0.462962955, 0.314814836},  //
      {-0.462962955, 0.351851851},  //
      {-0.42592594, 0.351851851},   //
      {-0.42592594, 0.351851851},   //
      {-0.42592594, 0.351851851},   //
  });
  TestPoly(polys, 454);
}

TEST(Polygon, Sponge4) {
  Polygons polys;
  polys.push_back({
      {0.166666672, 0.166666672},   //
      {0.166666672, 0.5},           //
      {0.150205776, 0.450617284},   //
      {0.154320985, 0.450617284},   //
      {0.154320985, 0.44650206},    //
      {0.154320985, 0.438271612},   //
      {0.150205761, 0.438271612},   //
      {0.146090552, 0.438271612},   //
      {0.117283955, 0.351851851},   //
      {0.129629627, 0.351851851},   //
      {0.129629627, 0.339506179},   //
      {0.129629627, 0.314814836},   //
      {0.117283948, 0.314814836},   //
      {0.104938284, 0.314814836},   //
      {0.076131694, 0.228395075},   //
      {0.0802469105, 0.228395075},  //
      {0.0802469105, 0.224279851},  //
      {0.0802469105, 0.216049403},  //
      {0.0761316866, 0.216049403},  //
      {0.0720164701, 0.216049403},  //
      {0.0555555597, 0.166666672},  //
      {0.166666672, 0.166666672},   //
  });
  polys.push_back({
      {-0.166666672, -0.5},            //
      {-0.117283955, -0.401234567},    //
      {-0.117283955, -0.401234567},    //
      {-0.117283955, -0.401234567},    //
      {-0.104938284, -0.376543224},    //
      {-0.1090535, -0.376543224},      //
      {-0.117283955, -0.376543224},    //
      {-0.117283955, -0.372428},       //
      {-0.117283955, -0.364197552},    //
      {-0.113168724, -0.364197552},    //
      {-0.104938269, -0.364197552},    //
      {-0.104938269, -0.368312776},    //
      {-0.104938269, -0.376543194},    //
      {-0.0308642201, -0.228395075},   //
      {-0.0349794254, -0.228395075},   //
      {-0.0432098769, -0.228395075},   //
      {-0.0432098769, -0.224279851},   //
      {-0.0432098769, -0.216049403},   //
      {-0.0390946493, -0.216049403},   //
      {-0.0308641978, -0.216049403},   //
      {-0.0308641978, -0.220164627},   //
      {-0.0308641978, -0.22839503},    //
      {-2.8974485e-09, -0.166666672},  //
      {-0.0555555597, -0.166666672},   //
      {-0.0720164701, -0.216049403},   //
      {-0.0679012388, -0.216049403},   //
      {-0.0679012388, -0.220164627},   //
      {-0.0679012388, -0.228395075},   //
      {-0.0720164627, -0.228395075},   //
      {-0.076131694, -0.228395075},    //
      {-0.104938284, -0.314814836},    //
      {-0.092592597, -0.314814836},    //
      {-0.092592597, -0.327160507},    //
      {-0.092592597, -0.351851851},    //
      {-0.104938276, -0.351851851},    //
      {-0.117283955, -0.351851851},    //
      {-0.146090552, -0.438271612},    //
      {-0.141975313, -0.438271612},    //
      {-0.141975313, -0.442386836},    //
      {-0.141975313, -0.450617284},    //
      {-0.146090537, -0.450617284},    //
      {-0.150205776, -0.450617284},    //
  });
  polys.push_back({
      {0.141975313, 0.179012358},  //
      {0.141975313, 0.183127582},  //
      {0.141975313, 0.19135803},   //
      {0.146090537, 0.19135803},   //
      {0.154320985, 0.19135803},   //
      {0.154320985, 0.187242806},  //
      {0.154320985, 0.179012358},  //
      {0.150205761, 0.179012358},  //
  });
  polys.push_back({
      {0.154320985, 0.216049403},  //
      {0.150205761, 0.216049403},  //
      {0.141975313, 0.216049403},  //
      {0.141975313, 0.220164627},  //
      {0.141975313, 0.228395075},  //
      {0.146090537, 0.228395075},  //
      {0.154320985, 0.228395075},  //
      {0.154320985, 0.224279851},  //
  });
  polys.push_back({
      {0.129629627, 0.203703716},  //
      {0.117283948, 0.203703716},  //
      {0.092592597, 0.203703716},  //
      {0.092592597, 0.216049403},  //
      {0.092592597, 0.240740761},  //
      {0.104938276, 0.240740761},  //
      {0.129629627, 0.240740761},  //
      {0.129629627, 0.228395075},  //
  });
  polys.push_back({
      {0.104938269, 0.179012358},  //
      {0.104938269, 0.183127582},  //
      {0.104938269, 0.19135803},   //
      {0.1090535, 0.19135803},     //
      {0.117283955, 0.19135803},   //
      {0.117283955, 0.187242806},  //
      {0.117283955, 0.179012358},  //
      {0.113168724, 0.179012358},  //
  });
  polys.push_back({
      {0.0679012388, 0.179012358},  //
      {0.0679012388, 0.183127582},  //
      {0.0679012388, 0.19135803},   //
      {0.0720164627, 0.19135803},   //
      {0.0802469105, 0.19135803},   //
      {0.0802469105, 0.187242806},  //
      {0.0802469105, 0.179012358},  //
      {0.0761316866, 0.179012358},  //
  });
  polys.push_back({
      {0.154320985, 0.253086448},  //
      {0.150205761, 0.253086448},  //
      {0.141975313, 0.253086448},  //
      {0.141975313, 0.257201672},  //
      {0.141975313, 0.265432119},  //
      {0.146090537, 0.265432119},  //
      {0.154320985, 0.265432119},  //
      {0.154320985, 0.261316895},  //
  });
  polys.push_back({
      {0.154320985, 0.290123463},  //
      {0.150205761, 0.290123463},  //
      {0.141975313, 0.290123463},  //
      {0.141975313, 0.294238687},  //
      {0.141975313, 0.302469134},  //
      {0.146090537, 0.302469134},  //
      {0.154320985, 0.302469134},  //
      {0.154320985, 0.29835391},   //
  });
  polys.push_back({
      {0.154320985, 0.327160507},  //
      {0.150205761, 0.327160507},  //
      {0.141975313, 0.327160507},  //
      {0.141975313, 0.331275731},  //
      {0.141975313, 0.339506179},  //
      {0.146090537, 0.339506179},  //
      {0.154320985, 0.339506179},  //
      {0.154320985, 0.335390955},  //
  });
  polys.push_back({
      {0.141975313, 0.364197552},  //
      {0.141975313, 0.368312776},  //
      {0.141975313, 0.376543224},  //
      {0.146090537, 0.376543224},  //
      {0.154320985, 0.376543224},  //
      {0.154320985, 0.372428},     //
      {0.154320985, 0.364197552},  //
      {0.150205761, 0.364197552},  //
  });
  polys.push_back({
      {0.154320985, 0.401234567},  //
      {0.150205761, 0.401234567},  //
      {0.141975313, 0.401234567},  //
      {0.141975313, 0.405349791},  //
      {0.141975313, 0.413580239},  //
      {0.146090537, 0.413580239},  //
      {0.154320985, 0.413580239},  //
      {0.154320985, 0.409465015},  //
  });
  polys.push_back({
      {0.117283955, 0.253086448},  //
      {0.113168724, 0.253086448},  //
      {0.104938269, 0.253086448},  //
      {0.104938269, 0.257201672},  //
      {0.104938269, 0.265432119},  //
      {0.1090535, 0.265432119},    //
      {0.117283955, 0.265432119},  //
      {0.117283955, 0.261316895},  //
  });
  polys.push_back({
      {0.117283955, 0.290123463},  //
      {0.113168724, 0.290123463},  //
      {0.104938269, 0.290123463},  //
      {0.104938269, 0.294238687},  //
      {0.104938269, 0.302469134},  //
      {0.1090535, 0.302469134},    //
      {0.117283955, 0.302469134},  //
      {0.117283955, 0.29835391},   //
  });
  polys.push_back({
      {-0.0679012388, -0.302469134},  //
      {-0.0720164627, -0.302469134},  //
      {-0.0802469105, -0.302469134},  //
      {-0.0802469105, -0.29835391},   //
      {-0.0802469105, -0.290123463},  //
      {-0.0761316866, -0.290123463},  //
      {-0.0679012388, -0.290123463},  //
      {-0.0679012388, -0.294238687},  //
  });
  polys.push_back({
      {-0.0679012388, -0.265432119},  //
      {-0.0720164627, -0.265432119},  //
      {-0.0802469105, -0.265432119},  //
      {-0.0802469105, -0.261316895},  //
      {-0.0802469105, -0.253086448},  //
      {-0.0761316866, -0.253086448},  //
      {-0.0679012388, -0.253086448},  //
      {-0.0679012388, -0.257201672},  //
  });
  polys.push_back({
      {-0.0308641978, -0.19135803},   //
      {-0.0349794254, -0.19135803},   //
      {-0.0432098769, -0.19135803},   //
      {-0.0432098769, -0.187242806},  //
      {-0.0432098769, -0.179012358},  //
      {-0.0390946493, -0.179012358},  //
      {-0.0308641978, -0.179012358},  //
      {-0.0308641978, -0.183127582},  //
  });
  TestPoly(polys, 210);
}

TEST(Polygon, BigSponge) {
  Polygons polys;
  polys.push_back({
      {0.5, 0.5},                        //
      {0.487654328, 0.4876543283834},    //
      {0.487654328, 0.4876543283869},    //
      {0.487654328, 0.4753086573868},    //
      {0.475308657, 0.4753086573870},    //
      {0.475308657, 0.4753086573835},    //
      {0.462962955, 0.4629629553833},    //
      {0.462962955, 0.425925943857},     //
      {0.42592594, 0.425925943830},      //
      {0.413580239, 0.4135802393832},    //
      {0.413580239, 0.4135802393861},    //
      {0.413580239, 0.4012345673862},    //
      {0.401234567, 0.4012345673863},    //
      {0.401234567, 0.4012345673831},    //
      {0.388888896, 0.3888888963836},    //
      {0.388888896, 0.3888888963826},    //
      {0.388888896, 0.2777777913834},    //
      {0.277777791, 0.2777777913842},    //
      {0.277777791, 0.2777777913829},    //
      {0.265432119, 0.2654321193827},    //
      {0.265432119, 0.2654321193840},    //
      {0.265432119, 0.2530864483839},    //
      {0.253086448, 0.2530864483841},    //
      {0.253086448, 0.2530864483828},    //
      {0.240740746, 0.2407407463824},    //
      {0.240740761, 0.2407407463783},    //
      {0.240740761, 0.2037037163781},    //
      {0.203703731, 0.2037037163784},    //
      {0.203703731, 0.2037037463825},    //
      {0.19135803, 0.191358033823},      //
      {0.19135803, 0.191358033782},      //
      {0.19135803, 0.1790123583776},     //
      {0.179012358, 0.1790123583785},    //
      {0.179012358, 0.1790123583826},    //
      {0.166666672, 0.1666666573822},    //
      {0.166666672, -0.1666666723546},   //
      {-0.166666672, -0.1666666723871},  //
      {-0.166666672, -0.1666666573837},  //
      {-0.179012358, -0.1790123583818},  //
      {-0.179012358, -0.191358033565},   //
      {-0.19135803, -0.191358033820},    //
      {-0.203703731, -0.2037037313821},  //
      {-0.203703716, -0.2037037313562},  //
      {-0.203703716, -0.2407407613563},  //
      {-0.240740746, -0.2407407613564},  //
      {-0.240740746, -0.2407407313819},  //
      {-0.253086448, -0.2530864483816},  //
      {-0.253086448, -0.2654321193519},  //
      {-0.265432119, -0.2654321193817},  //
      {-0.277777791, -0.2777777913815},  //
      {-0.277777791, -0.3888888963485},  //
      {-0.388888896, -0.3888888963808},  //
      {-0.401234567, -0.4012345673814},  //
      {-0.401234567, -0.4135802393509},  //
      {-0.413580239, -0.4135802393813},  //
      {-0.42592594, -0.425925943809},    //
      {-0.42592594, -0.4629629553504},   //
      {-0.462962955, -0.4629629553810},  //
      {-0.475308657, -0.4753086573811},  //
      {-0.475308657, -0.4876543283505},  //
      {-0.487654328, -0.4876543283812},  //
      {-0.5, -0.5},                      //
      {0.5, -0.5},                       //
  });
  polys.push_back({
      {0.487654328, -0.4876543282702},  //
      {0.475308657, -0.4876543282704},  //
      {0.475308657, -0.4753086572705},  //
      {0.487654328, -0.4753086572703},  //
  });
  polys.push_back({
      {0.450617284, -0.4753086572706},  //
      {0.450617284, -0.4876543282707},  //
      {0.438271612, -0.4876543282709},  //
      {0.438271612, -0.4753086572708},  //
  });
  polys.push_back({
      {0.462962955, -0.4629629552710},  //
      {0.42592591, -0.4629629552723},   //
      {0.42592591, -0.425925912735},    //
      {0.462962955, -0.425925912714},   //
  });
  polys.push_back({
      {0.487654328, -0.4506172842711},  //
      {0.475308657, -0.4506172842713},  //
      {0.475308657, -0.4382716122715},  //
      {0.487654328, -0.4382716122712},  //
  });
  polys.push_back({
      {0.413580239, -0.4753086572716},  //
      {0.413580239, -0.4876543282717},  //
      {0.401234567, -0.4876543282718},  //
      {0.401234567, -0.4753086572719},  //
  });
  polys.push_back({
      {0.376543224, -0.4753086572720},  //
      {0.376543224, -0.4876543282721},  //
      {0.364197552, -0.4876543282756},  //
      {0.364197552, -0.4753086572722},  //
  });
  polys.push_back({
      {0.413580239, -0.4382716122724},  //
      {0.413580239, -0.4506172842725},  //
      {0.401234567, -0.4506172842726},  //
      {0.401234567, -0.4382716122727},  //
  });
  polys.push_back({
      {0.376543224, -0.4506172842728},  //
      {0.364197552, -0.4506172842762},  //
      {0.364197552, -0.4382716122730},  //
      {0.376543224, -0.4382716122729},  //
  });
  polys.push_back({
      {0.487654328, -0.4135802392731},  //
      {0.475308657, -0.4135802392733},  //
      {0.475308657, -0.4012345672734},  //
      {0.487654328, -0.4012345672732},  //
  });
  polys.push_back({
      {0.450617284, -0.4135802392736},  //
      {0.438271612, -0.4135802392739},  //
      {0.438271612, -0.4012345672738},  //
      {0.450617284, -0.4012345672737},  //
  });
  polys.push_back({
      {0.487654328, -0.3765432242740},  //
      {0.475308657, -0.3765432242742},  //
      {0.475308657, -0.3641975522799},  //
      {0.487654328, -0.3641975522741},  //
  });
  polys.push_back({
      {0.450617284, -0.3765432242743},  //
      {0.438271612, -0.3765432242745},  //
      {0.438271612, -0.3641975522801},  //
      {0.450617284, -0.3641975522744},  //
  });
  polys.push_back({
      {0.413580239, -0.4135802392746},  //
      {0.401234567, -0.4135802392748},  //
      {0.401234567, -0.4012345672749},  //
      {0.413580239, -0.4012345672747},  //
  });
  polys.push_back({
      {0.376543224, -0.4135802392750},  //
      {0.364197552, -0.4135802392781},  //
      {0.364197552, -0.4012345672752},  //
      {0.376543224, -0.4012345672751},  //
  });
  polys.push_back({
      {0.413580239, -0.3641975522753},  //
      {0.413580239, -0.3765432242754},  //
      {0.401234567, -0.3765432242755},  //
      {0.401234567, -0.3641975522808},  //
  });
  polys.push_back({
      {0.339506179, -0.4753086572757},  //
      {0.339506179, -0.4876543282758},  //
      {0.327160507, -0.4876543282759},  //
      {0.327160507, -0.4753086572760},  //
  });
  polys.push_back({
      {0.351851851, -0.4629629552761},  //
      {0.314814806, -0.4629629552764},  //
      {0.314814806, -0.425925912783},   //
      {0.351851851, -0.425925912763},   //
  });
  polys.push_back({
      {0.302469134, -0.4753086572765},  //
      {0.302469134, -0.4876543282766},  //
      {0.290123463, -0.4876543282767},  //
      {0.290123463, -0.4753086572768},  //
  });
  polys.push_back({
      {0.265432119, -0.4876543282769},  //
      {0.253086448, -0.4876543282772},  //
      {0.253086448, -0.4753086572771},  //
      {0.265432119, -0.4753086572770},  //
  });
  polys.push_back({
      {0.302469134, -0.4506172842773},  //
      {0.290123463, -0.4506172842775},  //
      {0.290123463, -0.4382716122776},  //
      {0.302469134, -0.4382716122774},  //
  });
  polys.push_back({
      {0.265432119, -0.4506172842777},  //
      {0.253086448, -0.4506172842779},  //
      {0.253086448, -0.4382716122780},  //
      {0.265432119, -0.4382716122778},  //
  });
  polys.push_back({
      {0.339506179, -0.4012345672782},  //
      {0.339506179, -0.4135802392784},  //
      {0.327160507, -0.4135802392785},  //
      {0.327160507, -0.4012345672786},  //
  });
  polys.push_back({
      {0.388888896, -0.3888888962787},  //
      {0.277777791, -0.3888888962844},  //
      {0.277777791, -0.2777777912848},  //
      {0.388888896, -0.2777777912814},  //
  });
  polys.push_back({
      {0.302469134, -0.4135802392788},  //
      {0.290123463, -0.4135802392790},  //
      {0.290123463, -0.4012345672791},  //
      {0.302469134, -0.4012345672789},  //
  });
  polys.push_back({
      {0.265432119, -0.4135802392792},  //
      {0.253086448, -0.4135802392795},  //
      {0.253086448, -0.4012345672794},  //
      {0.265432119, -0.4012345672793},  //
  });
  polys.push_back({
      {0.265432119, -0.3765432242796},  //
      {0.253086448, -0.3765432242798},  //
      {0.253086448, -0.3641975522842},  //
      {0.265432119, -0.3641975522797},  //
  });
  polys.push_back({
      {0.487654328, -0.3395061792800},  //
      {0.475308657, -0.3395061792804},  //
      {0.475308657, -0.3271605072805},  //
      {0.487654328, -0.3271605072803},  //
  });
  polys.push_back({
      {0.462962955, -0.3518518512802},  //
      {0.42592591, -0.3518518512810},   //
      {0.42592591, -0.3148148062807},   //
      {0.462962955, -0.3148148062806},  //
  });
  polys.push_back({
      {0.413580239, -0.3395061792809},  //
      {0.401234567, -0.3395061792812},  //
      {0.401234567, -0.3271605072813},  //
      {0.413580239, -0.3271605072811},  //
  });
  polys.push_back({
      {0.487654328, -0.3024691342815},  //
      {0.475308657, -0.3024691342818},  //
      {0.475308657, -0.2901234632817},  //
      {0.487654328, -0.2901234632816},  //
  });
  polys.push_back({
      {0.450617284, -0.3024691342819},  //
      {0.438271612, -0.3024691342822},  //
      {0.438271612, -0.2901234632821},  //
      {0.450617284, -0.2901234632820},  //
  });
  polys.push_back({
      {0.487654328, -0.2654321192823},  //
      {0.475308657, -0.2654321192825},  //
      {0.475308657, -0.2530864482826},  //
      {0.487654328, -0.2530864482824},  //
  });
  polys.push_back({
      {0.450617284, -0.2530864482827},  //
      {0.450617284, -0.2654321192828},  //
      {0.438271612, -0.2654321192829},  //
      {0.438271612, -0.2530864482830},  //
  });
  polys.push_back({
      {0.413580239, -0.2901234632831},  //
      {0.413580239, -0.3024691342832},  //
      {0.401234567, -0.3024691342834},  //
      {0.401234567, -0.2901234632833},  //
  });
  polys.push_back({
      {0.413580239, -0.2530864482835},  //
      {0.413580239, -0.2654321192836},  //
      {0.401234567, -0.2654321192837},  //
      {0.401234567, -0.2530864482838},  //
  });
  polys.push_back({
      {0.376543224, -0.2530864482839},  //
      {0.376543224, -0.2654321192840},  //
      {0.364197552, -0.2654321192849},  //
      {0.364197552, -0.2530864482841},  //
  });
  polys.push_back({
      {0.265432119, -0.3395061792843},  //
      {0.253086448, -0.3395061792847},  //
      {0.253086448, -0.3271605072846},  //
      {0.265432119, -0.3271605072845},  //
  });
  polys.push_back({
      {0.339506179, -0.2530864482850},  //
      {0.339506179, -0.2654321192851},  //
      {0.327160507, -0.2654321192852},  //
      {0.327160507, -0.2530864482853},  //
  });
  polys.push_back({
      {0.265432119, -0.3024691342854},  //
      {0.253086448, -0.3024691342857},  //
      {0.253086448, -0.2901234632856},  //
      {0.265432119, -0.2901234632855},  //
  });
  polys.push_back({
      {0.302469134, -0.2530864482858},  //
      {0.302469134, -0.2654321192859},  //
      {0.290123463, -0.2654321192860},  //
      {0.290123463, -0.2530864482861},  //
  });
  polys.push_back({
      {0.265432119, -0.2654321192862},  //
      {0.253086448, -0.2654321192864},  //
      {0.253086448, -0.2530864482865},  //
      {0.265432119, -0.2530864482863},  //
  });
  polys.push_back({
      {0.228395075, -0.4876543282866},  //
      {0.216049403, -0.4876543282868},  //
      {0.216049403, -0.4753086572869},  //
      {0.228395075, -0.4753086572867},  //
  });
  polys.push_back({
      {0.240740761, -0.4629629552870},  //
      {0.203703731, -0.4629629552875},  //
      {0.203703731, -0.425925912890},   //
      {0.240740761, -0.425925912874},   //
  });
  polys.push_back({
      {0.19135803, -0.4753086572871},   //
      {0.19135803, -0.4876543282872},   //
      {0.179012358, -0.4876543282879},  //
      {0.179012358, -0.4753086572873},  //
  });
  polys.push_back({
      {0.19135803, -0.4506172842876},   //
      {0.179012358, -0.4506172842884},  //
      {0.179012358, -0.4382716122878},  //
      {0.19135803, -0.4382716122877},   //
  });
  polys.push_back({
      {0.154320985, -0.4753086572880},  //
      {0.154320985, -0.4876543282881},  //
      {0.141975313, -0.4876543282882},  //
      {0.141975313, -0.4753086572883},  //
  });
  polys.push_back({
      {0.154320985, -0.4382716122885},  //
      {0.154320985, -0.4506172842886},  //
      {0.141975313, -0.4506172842887},  //
      {0.141975313, -0.4382716122889},  //
  });
  polys.push_back({
      {0.129629627, -0.425925912888},    //
      {0.129629627, -0.4629629552916},   //
      {0.0925925896, -0.4629629552921},  //
      {0.0925925896, -0.425925912936},   //
  });
  polys.push_back({
      {0.228395075, -0.4135802392891},  //
      {0.216049403, -0.4135802392893},  //
      {0.216049403, -0.4012345672894},  //
      {0.228395075, -0.4012345672892},  //
  });
  polys.push_back({
      {0.19135803, -0.4135802392895},   //
      {0.179012358, -0.4135802392903},  //
      {0.179012358, -0.4012345672897},  //
      {0.19135803, -0.4012345672896},   //
  });
  polys.push_back({
      {0.228395075, -0.3765432242898},  //
      {0.216049403, -0.3765432242900},  //
      {0.216049403, -0.3641975522956},  //
      {0.228395075, -0.3641975522899},  //
  });
  polys.push_back({
      {0.19135803, -0.3765432242901},   //
      {0.179012358, -0.3765432242908},  //
      {0.179012358, -0.3641975522958},  //
      {0.19135803, -0.3641975522902},   //
  });
  polys.push_back({
      {0.154320985, -0.4012345672904},  //
      {0.154320985, -0.4135802392905},  //
      {0.141975313, -0.4135802392906},  //
      {0.141975313, -0.4012345672907},  //
  });
  polys.push_back({
      {0.154320985, -0.3641975522909},  //
      {0.154320985, -0.3765432242910},  //
      {0.141975313, -0.3765432242911},  //
      {0.141975313, -0.3641975522965},  //
  });
  polys.push_back({
      {0.117283955, -0.4753086572912},  //
      {0.117283955, -0.4876543282913},  //
      {0.104938276, -0.4876543282915},  //
      {0.104938276, -0.4753086572914},  //
  });
  polys.push_back({
      {0.0802469105, -0.4753086572917},  //
      {0.0802469105, -0.4876543282918},  //
      {0.0679012313, -0.4876543282919},  //
      {0.0679012313, -0.4753086572920},  //
  });
  polys.push_back({
      {0.0802469105, -0.4382716122922},  //
      {0.0802469105, -0.4506172842923},  //
      {0.0679012313, -0.4506172842924},  //
      {0.0679012313, -0.4382716122925},  //
  });
  polys.push_back({
      {0.0432098769, -0.4753086572926},  //
      {0.0432098769, -0.4876543282927},  //
      {0.0308641978, -0.4876543282928},  //
      {0.0308641978, -0.4753086572929},  //
  });
  polys.push_back({
      {0.00617283955, -0.4753086572930},   //
      {0.00617283955, -0.4876543283272},   //
      {-0.00617283955, -0.4876543283273},  //
      {-0.00617283955, -0.4753086573274},  //
  });
  polys.push_back({
      {0.0432098769, -0.4506172842931},  //
      {0.0308641978, -0.4506172842933},  //
      {0.0308641978, -0.4382716122934},  //
      {0.0432098769, -0.4382716122932},  //
  });
  polys.push_back({
      {0.0185185187, -0.425925912935},    //
      {0.0185185187, -0.4629629553275},   //
      {-0.0185185187, -0.4629629553280},  //
      {-0.0185185187, -0.425925913299},   //
  });
  polys.push_back({
      {0.117283955, -0.4135802392937},  //
      {0.104938276, -0.4135802392940},  //
      {0.104938276, -0.4012345672939},  //
      {0.117283955, -0.4012345672938},  //
  });
  polys.push_back({
      {0.0802469105, -0.4135802392941},  //
      {0.0679012313, -0.4135802392943},  //
      {0.0679012313, -0.4012345672944},  //
      {0.0802469105, -0.4012345672942},  //
  });
  polys.push_back({
      {0.117283955, -0.3765432242945},  //
      {0.104938276, -0.3765432242947},  //
      {0.104938276, -0.3641975522996},  //
      {0.117283955, -0.3641975522946},  //
  });
  polys.push_back({
      {0.0802469105, -0.3641975522948},  //
      {0.0802469105, -0.3765432242949},  //
      {0.0679012313, -0.3765432242950},  //
      {0.0679012313, -0.3641975522998},  //
  });
  polys.push_back({
      {0.0432098769, -0.4135802392951},  //
      {0.0308641978, -0.4135802392953},  //
      {0.0308641978, -0.4012345672954},  //
      {0.0432098769, -0.4012345672952},  //
  });
  polys.push_back({
      {0.00617283955, -0.4012345672955},   //
      {0.00617283955, -0.4135802393300},   //
      {-0.00617283955, -0.4135802393301},  //
      {-0.00617283955, -0.4012345673302},  //
  });
  polys.push_back({
      {0.240740761, -0.3518518512957},  //
      {0.203703731, -0.3518518512962},  //
      {0.203703731, -0.3148148062961},  //
      {0.240740761, -0.3148148062960},  //
  });
  polys.push_back({
      {0.19135803, -0.3395061792959},   //
      {0.179012358, -0.3395061792967},  //
      {0.179012358, -0.3271605072964},  //
      {0.19135803, -0.3271605072963},   //
  });
  polys.push_back({
      {0.154320985, -0.3395061792966},  //
      {0.141975313, -0.3395061792969},  //
      {0.141975313, -0.3271605072970},  //
      {0.154320985, -0.3271605072968},  //
  });
  polys.push_back({
      {0.129629627, -0.3148148062971},   //
      {0.129629627, -0.3518518512997},   //
      {0.0925925896, -0.3518518513000},  //
      {0.0925925896, -0.3148148063001},  //
  });
  polys.push_back({
      {0.228395075, -0.3024691342972},  //
      {0.216049403, -0.3024691342975},  //
      {0.216049403, -0.2901234632974},  //
      {0.228395075, -0.2901234632973},  //
  });
  polys.push_back({
      {0.19135803, -0.3024691342976},   //
      {0.179012358, -0.3024691342986},  //
      {0.179012358, -0.2901234632978},  //
      {0.19135803, -0.2901234632977},   //
  });
  polys.push_back({
      {0.228395075, -0.2654321192979},  //
      {0.216049403, -0.2654321192981},  //
      {0.216049403, -0.2530864482982},  //
      {0.228395075, -0.2530864482980},  //
  });
  polys.push_back({
      {0.19135803, -0.2530864482983},   //
      {0.19135803, -0.2654321192984},   //
      {0.179012358, -0.2654321192991},  //
      {0.179012358, -0.2530864482985},  //
  });
  polys.push_back({
      {0.154320985, -0.2901234632987},  //
      {0.154320985, -0.3024691342988},  //
      {0.141975313, -0.3024691342990},  //
      {0.141975313, -0.2901234632989},  //
  });
  polys.push_back({
      {0.154320985, -0.2530864482992},  //
      {0.154320985, -0.2654321192993},  //
      {0.141975313, -0.2654321192994},  //
      {0.141975313, -0.2530864482995},  //
  });
  polys.push_back({
      {0.0802469105, -0.3395061792999},  //
      {0.0679012313, -0.3395061793003},  //
      {0.0679012313, -0.3271605073004},  //
      {0.0802469105, -0.3271605073002},  //
  });
  polys.push_back({
      {0.055555556, -0.2777777913005},   //
      {0.055555556, -0.3888888963307},   //
      {-0.055555556, -0.3888888963369},  //
      {-0.055555556, -0.2777777913379},  //
  });
  polys.push_back({
      {0.117283955, -0.3024691343006},  //
      {0.104938276, -0.3024691343009},  //
      {0.104938276, -0.2901234633008},  //
      {0.117283955, -0.2901234633007},  //
  });
  polys.push_back({
      {0.0802469105, -0.2901234633010},  //
      {0.0802469105, -0.3024691343011},  //
      {0.0679012313, -0.3024691343013},  //
      {0.0679012313, -0.2901234633012},  //
  });
  polys.push_back({
      {0.117283955, -0.2530864483014},  //
      {0.117283955, -0.2654321193015},  //
      {0.104938276, -0.2654321193016},  //
      {0.104938276, -0.2530864483017},  //
  });
  polys.push_back({
      {0.0802469105, -0.2530864483018},  //
      {0.0802469105, -0.2654321193019},  //
      {0.0679012313, -0.2654321193020},  //
      {0.0679012313, -0.2530864483021},  //
  });
  polys.push_back({
      {0.0432098769, -0.2530864483022},  //
      {0.0432098769, -0.2654321193023},  //
      {0.0308641978, -0.2654321193024},  //
      {0.0308641978, -0.2530864483025},  //
  });
  polys.push_back({
      {0.00617283955, -0.2530864483026},   //
      {0.00617283955, -0.2654321193380},   //
      {-0.00617283955, -0.2654321193381},  //
      {-0.00617283955, -0.2530864483382},  //
  });
  polys.push_back({
      {0.487654328, -0.2283950753027},  //
      {0.475308657, -0.2283950753029},  //
      {0.475308657, -0.2160494033030},  //
      {0.487654328, -0.2160494033028},  //
  });
  polys.push_back({
      {0.462962955, -0.2037037313031},  //
      {0.462962955, -0.2407407613032},  //
      {0.42592591, -0.2407407613040},   //
      {0.42592591, -0.2037037313036},   //
  });
  polys.push_back({
      {0.487654328, -0.191358033033},   //
      {0.475308657, -0.191358033035},   //
      {0.475308657, -0.1790123583053},  //
      {0.487654328, -0.1790123583034},  //
  });
  polys.push_back({
      {0.450617284, -0.191358033037},   //
      {0.438271612, -0.191358033039},   //
      {0.438271612, -0.1790123583055},  //
      {0.450617284, -0.1790123583038},  //
  });
  polys.push_back({
      {0.413580239, -0.2160494033041},  //
      {0.413580239, -0.2283950753042},  //
      {0.401234567, -0.2283950753043},  //
      {0.401234567, -0.2160494033044},  //
  });
  polys.push_back({
      {0.376543224, -0.2160494033045},  //
      {0.376543224, -0.2283950753046},  //
      {0.364197552, -0.2283950753073},  //
      {0.364197552, -0.2160494033047},  //
  });
  polys.push_back({
      {0.413580239, -0.1790123583048},  //
      {0.413580239, -0.191358033049},   //
      {0.401234567, -0.191358033050},   //
      {0.401234567, -0.1790123583064},  //
  });
  polys.push_back({
      {0.376543224, -0.191358033051},   //
      {0.364197552, -0.191358033077},   //
      {0.364197552, -0.1790123583066},  //
      {0.376543224, -0.1790123583052},  //
  });
  polys.push_back({
      {0.487654328, -0.1543209853054},  //
      {0.475308657, -0.1543209853058},  //
      {0.475308657, -0.1419753133059},  //
      {0.487654328, -0.1419753133057},  //
  });
  polys.push_back({
      {0.450617284, -0.1543209853056},  //
      {0.438271612, -0.1543209853062},  //
      {0.438271612, -0.1419753133061},  //
      {0.450617284, -0.1419753133060},  //
  });
  polys.push_back({
      {0.462962955, -0.1296296273063},   //
      {0.42592591, -0.1296296273127},    //
      {0.42592591, -0.09259258963118},   //
      {0.462962955, -0.09259258963116},  //
  });
  polys.push_back({
      {0.413580239, -0.1543209853065},  //
      {0.401234567, -0.1543209853069},  //
      {0.401234567, -0.1419753133070},  //
      {0.413580239, -0.1419753133068},  //
  });
  polys.push_back({
      {0.376543224, -0.1543209853067},  //
      {0.364197552, -0.1543209853098},  //
      {0.364197552, -0.1419753133072},  //
      {0.376543224, -0.1419753133071},  //
  });
  polys.push_back({
      {0.351851851, -0.2037037313074},  //
      {0.351851851, -0.2407407613075},  //
      {0.314814806, -0.2407407613076},  //
      {0.314814806, -0.2037037313079},  //
  });
  polys.push_back({
      {0.339506179, -0.1790123583078},  //
      {0.339506179, -0.191358033080},   //
      {0.327160507, -0.191358033081},   //
      {0.327160507, -0.1790123583096},  //
  });
  polys.push_back({
      {0.302469134, -0.2160494033082},  //
      {0.302469134, -0.2283950753083},  //
      {0.290123463, -0.2283950753084},  //
      {0.290123463, -0.2160494033085},  //
  });
  polys.push_back({
      {0.265432119, -0.2283950753086},  //
      {0.253086448, -0.2283950753089},  //
      {0.253086448, -0.2160494033088},  //
      {0.265432119, -0.2160494033087},  //
  });
  polys.push_back({
      {0.302469134, -0.191358033090},   //
      {0.290123463, -0.191358033092},   //
      {0.290123463, -0.1790123583103},  //
      {0.302469134, -0.1790123583091},  //
  });
  polys.push_back({
      {0.265432119, -0.191358033093},   //
      {0.253086448, -0.191358033095},   //
      {0.253086448, -0.1790123583105},  //
      {0.265432119, -0.1790123583094},  //
  });
  polys.push_back({
      {0.339506179, -0.1543209853097},  //
      {0.327160507, -0.1543209853100},  //
      {0.327160507, -0.1419753133101},  //
      {0.339506179, -0.1419753133099},  //
  });
  polys.push_back({
      {0.351851851, -0.1296296273102},   //
      {0.314814806, -0.1296296273159},   //
      {0.314814806, -0.09259258963160},  //
      {0.351851851, -0.09259258963158},  //
  });
  polys.push_back({
      {0.302469134, -0.1543209853104},  //
      {0.290123463, -0.1543209853108},  //
      {0.290123463, -0.1419753133109},  //
      {0.302469134, -0.1419753133107},  //
  });
  polys.push_back({
      {0.265432119, -0.1543209853106},  //
      {0.253086448, -0.1543209853112},  //
      {0.253086448, -0.1419753133111},  //
      {0.265432119, -0.1419753133110},  //
  });
  polys.push_back({
      {0.487654328, -0.1172839553113},  //
      {0.475308657, -0.1172839553115},  //
      {0.475308657, -0.1049382763117},  //
      {0.487654328, -0.1049382763114},  //
  });
  polys.push_back({
      {0.487654328, -0.08024691053119},  //
      {0.475308657, -0.08024691053121},  //
      {0.475308657, -0.06790123133122},  //
      {0.487654328, -0.06790123133120},  //
  });
  polys.push_back({
      {0.450617284, -0.08024691053123},  //
      {0.438271612, -0.08024691053126},  //
      {0.438271612, -0.06790123133125},  //
      {0.450617284, -0.06790123133124},  //
  });
  polys.push_back({
      {0.413580239, -0.1049382763128},  //
      {0.413580239, -0.1172839553129},  //
      {0.401234567, -0.1172839553130},  //
      {0.401234567, -0.1049382763131},  //
  });
  polys.push_back({
      {0.376543224, -0.1172839553132},  //
      {0.364197552, -0.1172839553157},  //
      {0.364197552, -0.1049382763134},  //
      {0.376543224, -0.1049382763133},  //
  });
  polys.push_back({
      {0.413580239, -0.08024691053135},  //
      {0.401234567, -0.08024691053137},  //
      {0.401234567, -0.06790123133138},  //
      {0.413580239, -0.06790123133136},  //
  });
  polys.push_back({
      {0.376543224, -0.08024691053139},  //
      {0.364197552, -0.08024691053161},  //
      {0.364197552, -0.06790123133141},  //
      {0.376543224, -0.06790123133140},  //
  });
  polys.push_back({
      {0.487654328, -0.04320987693142},  //
      {0.475308657, -0.04320987693145},  //
      {0.475308657, -0.03086419783144},  //
      {0.487654328, -0.03086419783143},  //
  });
  polys.push_back({
      {0.450617284, -0.04320987693146},  //
      {0.438271612, -0.04320987693149},  //
      {0.438271612, -0.03086419783148},  //
      {0.450617284, -0.03086419783147},  //
  });
  polys.push_back({
      {0.487654328, -0.006172839553150},  //
      {0.475308657, -0.006172839553569},  //
      {0.475308657, 0.006172839553568},   //
      {0.487654328, 0.006172839553567},   //
  });
  polys.push_back({
      {0.462962955, -0.01851851873151},  //
      {0.42592591, -0.01851851873580},   //
      {0.42592591, 0.01851851873571},    //
      {0.462962955, 0.01851851873570},   //
  });
  polys.push_back({
      {0.413580239, -0.03086419783152},  //
      {0.413580239, -0.04320987693153},  //
      {0.401234567, -0.04320987693155},  //
      {0.401234567, -0.03086419783154},  //
  });
  polys.push_back({
      {0.413580239, -0.006172839553156},  //
      {0.401234567, -0.006172839553583},  //
      {0.401234567, 0.006172839553582},   //
      {0.413580239, 0.006172839553581},   //
  });
  polys.push_back({
      {0.339506179, -0.06790123133162},  //
      {0.339506179, -0.08024691053163},  //
      {0.327160507, -0.08024691053164},  //
      {0.327160507, -0.06790123133165},  //
  });
  polys.push_back({
      {0.302469134, -0.1172839553166},  //
      {0.290123463, -0.1172839553168},  //
      {0.290123463, -0.1049382763169},  //
      {0.302469134, -0.1049382763167},  //
  });
  polys.push_back({
      {0.265432119, -0.1172839553170},  //
      {0.253086448, -0.1172839553172},  //
      {0.253086448, -0.1049382763173},  //
      {0.265432119, -0.1049382763171},  //
  });
  polys.push_back({
      {0.302469134, -0.08024691053174},  //
      {0.290123463, -0.08024691053176},  //
      {0.290123463, -0.06790123133177},  //
      {0.302469134, -0.06790123133175},  //
  });
  polys.push_back({
      {0.265432119, -0.08024691053178},  //
      {0.253086448, -0.08024691053181},  //
      {0.253086448, -0.06790123133180},  //
      {0.265432119, -0.06790123133179},  //
  });
  polys.push_back({
      {0.388888896, -0.0555555563182},  //
      {0.277777791, -0.0555555563619},  //
      {0.277777791, 0.0555555563618},   //
      {0.388888896, 0.0555555563584},   //
  });
  polys.push_back({
      {0.265432119, -0.04320987693183},  //
      {0.253086448, -0.04320987693186},  //
      {0.253086448, -0.03086419783185},  //
      {0.265432119, -0.03086419783184},  //
  });
  polys.push_back({
      {0.265432119, -0.006172839553187},  //
      {0.253086448, -0.006172839553622},  //
      {0.253086448, 0.006172839553621},   //
      {0.265432119, 0.006172839553620},   //
  });
  polys.push_back({
      {0.240740761, -0.2407407613188},  //
      {0.203703731, -0.2407407613190},  //
      {0.203703731, -0.2037037313194},  //
      {0.240740761, -0.2037037313189},  //
  });
  polys.push_back({
      {0.19135803, -0.2160494033191},   //
      {0.19135803, -0.2283950753192},   //
      {0.179012358, -0.2283950753200},  //
      {0.179012358, -0.2160494033193},  //
  });
  polys.push_back({
      {0.228395075, -0.191358033195},   //
      {0.216049403, -0.191358033197},   //
      {0.216049403, -0.1790123583210},  //
      {0.228395075, -0.1790123583196},  //
  });
  polys.push_back({
      {0.19135803, -0.191358033198},    //
      {0.179012358, -0.191358033206},   //
      {0.179012358, -0.1790123583212},  //
      {0.19135803, -0.1790123583199},   //
  });
  polys.push_back({
      {0.154320985, -0.2160494033201},  //
      {0.154320985, -0.2283950753202},  //
      {0.141975313, -0.2283950753203},  //
      {0.141975313, -0.2160494033204},  //
  });
  polys.push_back({
      {0.129629627, -0.2037037313205},   //
      {0.129629627, -0.2407407613222},   //
      {0.0925925896, -0.2407407613223},  //
      {0.0925925896, -0.2037037313228},  //
  });
  polys.push_back({
      {0.154320985, -0.1790123583207},  //
      {0.154320985, -0.191358033208},   //
      {0.141975313, -0.191358033209},   //
      {0.141975313, -0.1790123583220},  //
  });
  polys.push_back({
      {0.228395075, -0.1543209853211},  //
      {0.216049403, -0.1543209853215},  //
      {0.216049403, -0.1419753133216},  //
      {0.228395075, -0.1419753133214},  //
  });
  polys.push_back({
      {0.19135803, -0.1543209853213},   //
      {0.179012358, -0.1543209853221},  //
      {0.179012358, -0.1419753133219},  //
      {0.19135803, -0.1419753133218},   //
  });
  polys.push_back({
      {0.240740761, -0.1296296273217},   //
      {0.203703731, -0.1296296273249},   //
      {0.203703731, -0.09259258963248},  //
      {0.240740761, -0.09259258963247},  //
  });
  polys.push_back({
      {0.0802469105, -0.2160494033224},  //
      {0.0802469105, -0.2283950753225},  //
      {0.0679012313, -0.2283950753226},  //
      {0.0679012313, -0.2160494033227},  //
  });
  polys.push_back({
      {0.117283955, -0.191358033229},   //
      {0.104938276, -0.191358033231},   //
      {0.104938276, -0.1790123583244},  //
      {0.117283955, -0.1790123583230},  //
  });
  polys.push_back({
      {0.0802469105, -0.1790123583232},  //
      {0.0802469105, -0.191358033233},   //
      {0.0679012313, -0.191358033234},   //
      {0.0679012313, -0.1790123583245},  //
  });
  polys.push_back({
      {0.0432098769, -0.2160494033235},  //
      {0.0432098769, -0.2283950753236},  //
      {0.0308641978, -0.2283950753237},  //
      {0.0308641978, -0.2160494033238},  //
  });
  polys.push_back({
      {0.0185185187, -0.2037037313239},   //
      {0.0185185187, -0.2407407613520},   //
      {-0.0185185187, -0.2407407613521},  //
      {-0.0185185187, -0.2037037313526},  //
  });
  polys.push_back({
      {0.0432098769, -0.191358033240},   //
      {0.0308641978, -0.191358033242},   //
      {0.0308641978, -0.1790123583246},  //
      {0.0432098769, -0.1790123583241},  //
  });
  polys.push_back({
      {0.00617283955, -0.1790123583243},   //
      {0.00617283955, -0.191358033527},    //
      {-0.00617283955, -0.191358033528},   //
      {-0.00617283955, -0.1790123583545},  //
  });
  polys.push_back({
      {0.19135803, -0.1172839553250},   //
      {0.179012358, -0.1172839553260},  //
      {0.179012358, -0.1049382763252},  //
      {0.19135803, -0.1049382763251},   //
  });
  polys.push_back({
      {0.228395075, -0.08024691053253},  //
      {0.216049403, -0.08024691053255},  //
      {0.216049403, -0.06790123133256},  //
      {0.228395075, -0.06790123133254},  //
  });
  polys.push_back({
      {0.19135803, -0.08024691053257},   //
      {0.179012358, -0.08024691053261},  //
      {0.179012358, -0.06790123133259},  //
      {0.19135803, -0.06790123133258},   //
  });
  polys.push_back({
      {0.228395075, -0.04320987693262},  //
      {0.216049403, -0.04320987693265},  //
      {0.216049403, -0.03086419783264},  //
      {0.228395075, -0.03086419783263},  //
  });
  polys.push_back({
      {0.19135803, -0.04320987693266},   //
      {0.179012358, -0.04320987693271},  //
      {0.179012358, -0.03086419783268},  //
      {0.19135803, -0.03086419783267},   //
  });
  polys.push_back({
      {0.240740761, -0.01851851873269},  //
      {0.203703731, -0.01851851873740},  //
      {0.203703731, 0.01851851873739},   //
      {0.240740761, 0.01851851873738},   //
  });
  polys.push_back({
      {0.19135803, -0.006172839553270},   //
      {0.179012358, -0.006172839553750},  //
      {0.179012358, 0.006172839553742},   //
      {0.19135803, 0.006172839553741},    //
  });
  polys.push_back({
      {-0.0308641978, -0.4876543283276},  //
      {-0.0432098769, -0.4876543283278},  //
      {-0.0432098769, -0.4753086573279},  //
      {-0.0308641978, -0.4753086573277},  //
  });
  polys.push_back({
      {-0.0308641978, -0.4506172843281},  //
      {-0.0432098769, -0.4506172843283},  //
      {-0.0432098769, -0.4382716123284},  //
      {-0.0308641978, -0.4382716123282},  //
  });
  polys.push_back({
      {-0.0679012388, -0.4753086573285},  //
      {-0.0679012388, -0.4876543283286},  //
      {-0.0802469179, -0.4876543283288},  //
      {-0.0802469179, -0.4753086573287},  //
  });
  polys.push_back({
      {-0.104938269, -0.4753086573289},  //
      {-0.104938269, -0.4876543283290},  //
      {-0.117283948, -0.4876543283291},  //
      {-0.117283948, -0.4753086573292},  //
  });
  polys.push_back({
      {-0.092592597, -0.4629629553293},  //
      {-0.129629642, -0.4629629553328},  //
      {-0.129629642, -0.425925913312},   //
      {-0.092592597, -0.425925913298},   //
  });
  polys.push_back({
      {-0.0679012388, -0.4506172843294},  //
      {-0.0802469179, -0.4506172843296},  //
      {-0.0802469179, -0.4382716123297},  //
      {-0.0679012388, -0.4382716123295},  //
  });
  polys.push_back({
      {-0.0308641978, -0.4135802393303},  //
      {-0.0432098769, -0.4135802393305},  //
      {-0.0432098769, -0.4012345673306},  //
      {-0.0308641978, -0.4012345673304},  //
  });
  polys.push_back({
      {-0.0679012388, -0.4135802393308},  //
      {-0.0802469179, -0.4135802393311},  //
      {-0.0802469179, -0.4012345673310},  //
      {-0.0679012388, -0.4012345673309},  //
  });
  polys.push_back({
      {-0.104938269, -0.4135802393313},  //
      {-0.117283948, -0.4135802393315},  //
      {-0.117283948, -0.4012345673316},  //
      {-0.104938269, -0.4012345673314},  //
  });
  polys.push_back({
      {-0.0679012388, -0.3765432243317},  //
      {-0.0802469179, -0.3765432243319},  //
      {-0.0802469179, -0.3641975523370},  //
      {-0.0679012388, -0.3641975523318},  //
  });
  polys.push_back({
      {-0.104938269, -0.3641975523320},  //
      {-0.104938269, -0.3765432243321},  //
      {-0.117283948, -0.3765432243322},  //
      {-0.117283948, -0.3641975523372},  //
  });
  polys.push_back({
      {-0.141975313, -0.4753086573323},  //
      {-0.141975313, -0.4876543283324},  //
      {-0.154320985, -0.4876543283326},  //
      {-0.154320985, -0.4753086573325},  //
  });
  polys.push_back({
      {-0.179012358, -0.4753086573327},  //
      {-0.179012358, -0.4876543283334},  //
      {-0.19135803, -0.4876543283335},   //
      {-0.19135803, -0.4753086573336},   //
  });
  polys.push_back({
      {-0.141975313, -0.4506172843329},  //
      {-0.154320985, -0.4506172843332},  //
      {-0.154320985, -0.4382716123331},  //
      {-0.141975313, -0.4382716123330},  //
  });
  polys.push_back({
      {-0.179012358, -0.4382716123333},  //
      {-0.179012358, -0.4506172843342},  //
      {-0.19135803, -0.4506172843343},   //
      {-0.19135803, -0.4382716123345},   //
  });
  polys.push_back({
      {-0.216049403, -0.4753086573337},  //
      {-0.216049403, -0.4876543283338},  //
      {-0.228395075, -0.4876543283339},  //
      {-0.228395075, -0.4753086573340},  //
  });
  polys.push_back({
      {-0.203703716, -0.4629629553341},  //
      {-0.240740746, -0.4629629553346},  //
      {-0.240740746, -0.425925913359},   //
      {-0.203703716, -0.425925913344},   //
  });
  polys.push_back({
      {-0.141975313, -0.4135802393347},  //
      {-0.154320985, -0.4135802393350},  //
      {-0.154320985, -0.4012345673349},  //
      {-0.141975313, -0.4012345673348},  //
  });
  polys.push_back({
      {-0.179012358, -0.4012345673351},  //
      {-0.179012358, -0.4135802393356},  //
      {-0.19135803, -0.4135802393357},   //
      {-0.19135803, -0.4012345673358},   //
  });
  polys.push_back({
      {-0.141975313, -0.3765432243352},  //
      {-0.154320985, -0.3765432243354},  //
      {-0.154320985, -0.3641975523403},  //
      {-0.141975313, -0.3641975523353},  //
  });
  polys.push_back({
      {-0.179012358, -0.3641975523355},  //
      {-0.179012358, -0.3765432243364},  //
      {-0.19135803, -0.3765432243365},   //
      {-0.19135803, -0.3641975523410},   //
  });
  polys.push_back({
      {-0.216049403, -0.4135802393360},  //
      {-0.228395075, -0.4135802393362},  //
      {-0.228395075, -0.4012345673363},  //
      {-0.216049403, -0.4012345673361},  //
  });
  polys.push_back({
      {-0.216049403, -0.3765432243366},  //
      {-0.228395075, -0.3765432243368},  //
      {-0.228395075, -0.3641975523412},  //
      {-0.216049403, -0.3641975523367},  //
  });
  polys.push_back({
      {-0.0679012388, -0.3395061793371},  //
      {-0.0802469179, -0.3395061793376},  //
      {-0.0802469179, -0.3271605073375},  //
      {-0.0679012388, -0.3271605073374},  //
  });
  polys.push_back({
      {-0.092592597, -0.3518518513373},  //
      {-0.129629642, -0.3518518513405},  //
      {-0.129629642, -0.3148148063378},  //
      {-0.092592597, -0.3148148063377},  //
  });
  polys.push_back({
      {-0.0308641978, -0.2654321193383},  //
      {-0.0432098769, -0.2654321193385},  //
      {-0.0432098769, -0.2530864483386},  //
      {-0.0308641978, -0.2530864483384},  //
  });
  polys.push_back({
      {-0.0679012388, -0.3024691343387},  //
      {-0.0802469179, -0.3024691343390},  //
      {-0.0802469179, -0.2901234633389},  //
      {-0.0679012388, -0.2901234633388},  //
  });
  polys.push_back({
      {-0.104938269, -0.2901234633391},  //
      {-0.104938269, -0.3024691343392},  //
      {-0.117283948, -0.3024691343394},  //
      {-0.117283948, -0.2901234633393},  //
  });
  polys.push_back({
      {-0.0679012388, -0.2530864483395},  //
      {-0.0679012388, -0.2654321193396},  //
      {-0.0802469179, -0.2654321193397},  //
      {-0.0802469179, -0.2530864483398},  //
  });
  polys.push_back({
      {-0.104938269, -0.2530864483399},  //
      {-0.104938269, -0.2654321193400},  //
      {-0.117283948, -0.2654321193401},  //
      {-0.117283948, -0.2530864483402},  //
  });
  polys.push_back({
      {-0.141975313, -0.3395061793404},  //
      {-0.154320985, -0.3395061793408},  //
      {-0.154320985, -0.3271605073407},  //
      {-0.141975313, -0.3271605073406},  //
  });
  polys.push_back({
      {-0.179012358, -0.3271605073409},  //
      {-0.179012358, -0.3395061793411},  //
      {-0.19135803, -0.3395061793414},   //
      {-0.19135803, -0.3271605073415},   //
  });
  polys.push_back({
      {-0.203703716, -0.3518518513413},  //
      {-0.240740746, -0.3518518513417},  //
      {-0.240740746, -0.3148148063418},  //
      {-0.203703716, -0.3148148063416},  //
  });
  polys.push_back({
      {-0.141975313, -0.3024691343419},  //
      {-0.154320985, -0.3024691343422},  //
      {-0.154320985, -0.2901234633421},  //
      {-0.141975313, -0.2901234633420},  //
  });
  polys.push_back({
      {-0.179012358, -0.2901234633423},  //
      {-0.179012358, -0.3024691343429},  //
      {-0.19135803, -0.3024691343431},   //
      {-0.19135803, -0.2901234633430},   //
  });
  polys.push_back({
      {-0.141975313, -0.2530864483424},  //
      {-0.141975313, -0.2654321193425},  //
      {-0.154320985, -0.2654321193427},  //
      {-0.154320985, -0.2530864483426},  //
  });
  polys.push_back({
      {-0.179012358, -0.2530864483428},  //
      {-0.179012358, -0.2654321193436},  //
      {-0.19135803, -0.2654321193437},   //
      {-0.19135803, -0.2530864483438},   //
  });
  polys.push_back({
      {-0.216049403, -0.3024691343432},  //
      {-0.228395075, -0.3024691343434},  //
      {-0.228395075, -0.2901234633435},  //
      {-0.216049403, -0.2901234633433},  //
  });
  polys.push_back({
      {-0.216049403, -0.2530864483439},  //
      {-0.216049403, -0.2654321193440},  //
      {-0.228395075, -0.2654321193441},  //
      {-0.228395075, -0.2530864483442},  //
  });
  polys.push_back({
      {-0.253086448, -0.4753086573443},  //
      {-0.253086448, -0.4876543283444},  //
      {-0.265432119, -0.4876543283445},  //
      {-0.265432119, -0.4753086573446},  //
  });
  polys.push_back({
      {-0.290123463, -0.4876543283447},  //
      {-0.302469134, -0.4876543283449},  //
      {-0.302469134, -0.4753086573450},  //
      {-0.290123463, -0.4753086573448},  //
  });
  polys.push_back({
      {-0.253086448, -0.4382716123451},  //
      {-0.253086448, -0.4506172843452},  //
      {-0.265432119, -0.4506172843453},  //
      {-0.265432119, -0.4382716123454},  //
  });
  polys.push_back({
      {-0.290123463, -0.4506172843455},  //
      {-0.302469134, -0.4506172843457},  //
      {-0.302469134, -0.4382716123458},  //
      {-0.290123463, -0.4382716123456},  //
  });
  polys.push_back({
      {-0.327160507, -0.4753086573459},  //
      {-0.327160507, -0.4876543283460},  //
      {-0.339506179, -0.4876543283463},  //
      {-0.339506179, -0.4753086573461},  //
  });
  polys.push_back({
      {-0.314814836, -0.4629629553462},  //
      {-0.351851881, -0.4629629553466},  //
      {-0.351851881, -0.425925913479},   //
      {-0.314814836, -0.425925913465},   //
  });
  polys.push_back({
      {-0.364197552, -0.4753086573464},  //
      {-0.364197552, -0.4876543283486},  //
      {-0.376543224, -0.4876543283487},  //
      {-0.376543224, -0.4753086573488},  //
  });
  polys.push_back({
      {-0.364197552, -0.4382716123467},  //
      {-0.364197552, -0.4506172843493},  //
      {-0.376543224, -0.4506172843494},  //
      {-0.376543224, -0.4382716123495},  //
  });
  polys.push_back({
      {-0.253086448, -0.4135802393468},  //
      {-0.265432119, -0.4135802393470},  //
      {-0.265432119, -0.4012345673471},  //
      {-0.253086448, -0.4012345673469},  //
  });
  polys.push_back({
      {-0.290123463, -0.4135802393472},  //
      {-0.302469134, -0.4135802393474},  //
      {-0.302469134, -0.4012345673475},  //
      {-0.290123463, -0.4012345673473},  //
  });
  polys.push_back({
      {-0.253086448, -0.3641975523476},  //
      {-0.253086448, -0.3765432243477},  //
      {-0.265432119, -0.3765432243478},  //
      {-0.265432119, -0.3641975523510},  //
  });
  polys.push_back({
      {-0.327160507, -0.4135802393480},  //
      {-0.339506179, -0.4135802393483},  //
      {-0.339506179, -0.4012345673482},  //
      {-0.327160507, -0.4012345673481},  //
  });
  polys.push_back({
      {-0.364197552, -0.4012345673484},  //
      {-0.364197552, -0.4135802393506},  //
      {-0.376543224, -0.4135802393507},  //
      {-0.376543224, -0.4012345673508},  //
  });
  polys.push_back({
      {-0.401234567, -0.4753086573489},  //
      {-0.401234567, -0.4876543283490},  //
      {-0.413580239, -0.4876543283492},  //
      {-0.413580239, -0.4753086573491},  //
  });
  polys.push_back({
      {-0.401234567, -0.4506172843496},  //
      {-0.413580239, -0.4506172843498},  //
      {-0.413580239, -0.4382716123499},  //
      {-0.401234567, -0.4382716123497},  //
  });
  polys.push_back({
      {-0.438271612, -0.4753086573500},  //
      {-0.438271612, -0.4876543283501},  //
      {-0.450617284, -0.4876543283502},  //
      {-0.450617284, -0.4753086573503},  //
  });
  polys.push_back({
      {-0.253086448, -0.3395061793511},  //
      {-0.265432119, -0.3395061793513},  //
      {-0.265432119, -0.3271605073514},  //
      {-0.253086448, -0.3271605073512},  //
  });
  polys.push_back({
      {-0.253086448, -0.2901234633515},  //
      {-0.253086448, -0.3024691343516},  //
      {-0.265432119, -0.3024691343517},  //
      {-0.265432119, -0.2901234633518},  //
  });
  polys.push_back({
      {-0.0308641978, -0.2283950753522},  //
      {-0.0432098769, -0.2283950753524},  //
      {-0.0432098769, -0.2160494033525},  //
      {-0.0308641978, -0.2160494033523},  //
  });
  polys.push_back({
      {-0.0308641978, -0.191358033529},   //
      {-0.0432098769, -0.191358033531},   //
      {-0.0432098769, -0.1790123583547},  //
      {-0.0308641978, -0.1790123583530},  //
  });
  polys.push_back({
      {-0.0679012388, -0.2160494033532},  //
      {-0.0679012388, -0.2283950753533},  //
      {-0.0802469179, -0.2283950753535},  //
      {-0.0802469179, -0.2160494033534},  //
  });
  polys.push_back({
      {-0.092592597, -0.2407407613536},  //
      {-0.129629642, -0.2407407613550},  //
      {-0.129629642, -0.2037037313541},  //
      {-0.092592597, -0.2037037313537},  //
  });
  polys.push_back({
      {-0.0679012388, -0.191358033538},   //
      {-0.0802469179, -0.191358033540},   //
      {-0.0802469179, -0.1790123583548},  //
      {-0.0679012388, -0.1790123583539},  //
  });
  polys.push_back({
      {-0.104938269, -0.1790123583542},  //
      {-0.104938269, -0.191358033543},   //
      {-0.117283948, -0.191358033544},   //
      {-0.117283948, -0.1790123583549},  //
  });
  polys.push_back({
      {-0.141975313, -0.2160494033551},  //
      {-0.141975313, -0.2283950753552},  //
      {-0.154320985, -0.2283950753554},  //
      {-0.154320985, -0.2160494033553},  //
  });
  polys.push_back({
      {-0.179012358, -0.2160494033555},  //
      {-0.179012358, -0.2283950753559},  //
      {-0.19135803, -0.2283950753560},   //
      {-0.19135803, -0.2160494033561},   //
  });
  polys.push_back({
      {-0.141975313, -0.191358033556},   //
      {-0.154320985, -0.191358033558},   //
      {-0.154320985, -0.1790123583566},  //
      {-0.141975313, -0.1790123583557},  //
  });
  polys.push_back({
      {0.487654328, 0.03086419783572},  //
      {0.475308657, 0.03086419783574},  //
      {0.475308657, 0.04320987693575},  //
      {0.487654328, 0.04320987693573},  //
  });
  polys.push_back({
      {0.450617284, 0.04320987693576},  //
      {0.450617284, 0.03086419783577},  //
      {0.438271612, 0.03086419783579},  //
      {0.438271612, 0.04320987693578},  //
  });
  polys.push_back({
      {0.413580239, 0.04320987693585},  //
      {0.413580239, 0.03086419783586},  //
      {0.401234567, 0.03086419783587},  //
      {0.401234567, 0.04320987693588},  //
  });
  polys.push_back({
      {0.487654328, 0.06790123883589},  //
      {0.475308657, 0.06790123883591},  //
      {0.475308657, 0.08024691793592},  //
      {0.487654328, 0.08024691793590},  //
  });
  polys.push_back({
      {0.450617284, 0.06790123883593},  //
      {0.438271612, 0.06790123883595},  //
      {0.438271612, 0.08024691793596},  //
      {0.450617284, 0.08024691793594},  //
  });
  polys.push_back({
      {0.487654328, 0.1049382693597},  //
      {0.475308657, 0.1049382693599},  //
      {0.475308657, 0.1172839483600},  //
      {0.487654328, 0.1172839483598},  //
  });
  polys.push_back({
      {0.462962955, 0.1296296423601},  //
      {0.462962955, 0.0925925973602},  //
      {0.42592591, 0.0925925973611},   //
      {0.42592591, 0.1296296423655},   //
  });
  polys.push_back({
      {0.413580239, 0.08024691793603},  //
      {0.413580239, 0.06790123883604},  //
      {0.401234567, 0.06790123883605},  //
      {0.401234567, 0.08024691793606},  //
  });
  polys.push_back({
      {0.376543224, 0.06790123883607},  //
      {0.364197552, 0.06790123883627},  //
      {0.364197552, 0.08024691793609},  //
      {0.376543224, 0.08024691793608},  //
  });
  polys.push_back({
      {0.413580239, 0.1049382693610},  //
      {0.401234567, 0.1049382693613},  //
      {0.401234567, 0.1172839483614},  //
      {0.413580239, 0.1172839483612},  //
  });
  polys.push_back({
      {0.376543224, 0.1049382693615},  //
      {0.364197552, 0.1049382693632},  //
      {0.364197552, 0.1172839483617},  //
      {0.376543224, 0.1172839483616},  //
  });
  polys.push_back({
      {0.265432119, 0.03086419783623},  //
      {0.253086448, 0.03086419783626},  //
      {0.253086448, 0.04320987693625},  //
      {0.265432119, 0.04320987693624},  //
  });
  polys.push_back({
      {0.339506179, 0.08024691793628},  //
      {0.339506179, 0.06790123883629},  //
      {0.327160507, 0.06790123883630},  //
      {0.327160507, 0.08024691793631},  //
  });
  polys.push_back({
      {0.351851851, 0.1296296423633},  //
      {0.351851851, 0.0925925973634},  //
      {0.314814806, 0.0925925973635},  //
      {0.314814806, 0.1296296423700},  //
  });
  polys.push_back({
      {0.302469134, 0.06790123883636},  //
      {0.290123463, 0.06790123883638},  //
      {0.290123463, 0.08024691793639},  //
      {0.302469134, 0.08024691793637},  //
  });
  polys.push_back({
      {0.265432119, 0.06790123883640},  //
      {0.253086448, 0.06790123883642},  //
      {0.253086448, 0.08024691793643},  //
      {0.265432119, 0.08024691793641},  //
  });
  polys.push_back({
      {0.302469134, 0.1049382693644},  //
      {0.290123463, 0.1049382693646},  //
      {0.290123463, 0.1172839483647},  //
      {0.302469134, 0.1172839483645},  //
  });
  polys.push_back({
      {0.265432119, 0.1049382693648},  //
      {0.253086448, 0.1049382693651},  //
      {0.253086448, 0.1172839483650},  //
      {0.265432119, 0.1172839483649},  //
  });
  polys.push_back({
      {0.487654328, 0.1419753133652},  //
      {0.475308657, 0.1419753133654},  //
      {0.475308657, 0.1543209853659},  //
      {0.487654328, 0.1543209853653},  //
  });
  polys.push_back({
      {0.450617284, 0.1419753133656},  //
      {0.438271612, 0.1419753133658},  //
      {0.438271612, 0.1543209853661},  //
      {0.450617284, 0.1543209853657},  //
  });
  polys.push_back({
      {0.487654328, 0.1790123583660},  //
      {0.475308657, 0.1790123583673},  //
      {0.475308657, 0.191358033674},   //
      {0.487654328, 0.191358033672},   //
  });
  polys.push_back({
      {0.450617284, 0.1790123583662},  //
      {0.438271612, 0.1790123583677},  //
      {0.438271612, 0.191358033676},   //
      {0.450617284, 0.191358033675},   //
  });
  polys.push_back({
      {0.413580239, 0.1543209853663},  //
      {0.413580239, 0.1419753133664},  //
      {0.401234567, 0.1419753133665},  //
      {0.401234567, 0.1543209853668},  //
  });
  polys.push_back({
      {0.376543224, 0.1419753133666},  //
      {0.364197552, 0.1419753133698},  //
      {0.364197552, 0.1543209853670},  //
      {0.376543224, 0.1543209853667},  //
  });
  polys.push_back({
      {0.413580239, 0.1790123583669},  //
      {0.401234567, 0.1790123583686},  //
      {0.401234567, 0.191358033687},   //
      {0.413580239, 0.191358033685},   //
  });
  polys.push_back({
      {0.376543224, 0.1790123583671},  //
      {0.364197552, 0.1790123583715},  //
      {0.364197552, 0.191358033689},   //
      {0.376543224, 0.191358033688},   //
  });
  polys.push_back({
      {0.462962955, 0.2037037163678},  //
      {0.42592591, 0.2037037163690},   //
      {0.42592591, 0.2407407463684},   //
      {0.462962955, 0.2407407463683},  //
  });
  polys.push_back({
      {0.487654328, 0.2160494033679},  //
      {0.475308657, 0.2160494033682},  //
      {0.475308657, 0.2283950753681},  //
      {0.487654328, 0.2283950753680},  //
  });
  polys.push_back({
      {0.413580239, 0.2283950753691},  //
      {0.413580239, 0.2160494033692},  //
      {0.401234567, 0.2160494033694},  //
      {0.401234567, 0.2283950753693},  //
  });
  polys.push_back({
      {0.376543224, 0.2160494033695},  //
      {0.364197552, 0.2160494033720},  //
      {0.364197552, 0.2283950753697},  //
      {0.376543224, 0.2283950753696},  //
  });
  polys.push_back({
      {0.339506179, 0.1543209853699},  //
      {0.339506179, 0.1419753133701},  //
      {0.327160507, 0.1419753133702},  //
      {0.327160507, 0.1543209853703},  //
  });
  polys.push_back({
      {0.339506179, 0.1790123583704},  //
      {0.327160507, 0.1790123583717},  //
      {0.327160507, 0.191358033718},   //
      {0.339506179, 0.191358033716},   //
  });
  polys.push_back({
      {0.302469134, 0.1419753133705},  //
      {0.290123463, 0.1419753133707},  //
      {0.290123463, 0.1543209853711},  //
      {0.302469134, 0.1543209853706},  //
  });
  polys.push_back({
      {0.265432119, 0.1419753133708},  //
      {0.253086448, 0.1419753133710},  //
      {0.253086448, 0.1543209853713},  //
      {0.265432119, 0.1543209853709},  //
  });
  polys.push_back({
      {0.302469134, 0.1790123583712},  //
      {0.290123463, 0.1790123583725},  //
      {0.290123463, 0.191358033726},   //
      {0.302469134, 0.191358033724},   //
  });
  polys.push_back({
      {0.265432119, 0.1790123583714},  //
      {0.253086448, 0.1790123583729},  //
      {0.253086448, 0.191358033728},   //
      {0.265432119, 0.191358033727},   //
  });
  polys.push_back({
      {0.351851851, 0.2037037163719},  //
      {0.314814806, 0.2037037163722},  //
      {0.314814806, 0.2407407463723},  //
      {0.351851851, 0.2407407463721},  //
  });
  polys.push_back({
      {0.302469134, 0.2160494033730},  //
      {0.290123463, 0.2160494033732},  //
      {0.290123463, 0.2283950753733},  //
      {0.302469134, 0.2283950753731},  //
  });
  polys.push_back({
      {0.265432119, 0.2160494033734},  //
      {0.253086448, 0.2160494033737},  //
      {0.253086448, 0.2283950753736},  //
      {0.265432119, 0.2283950753735},  //
  });
  polys.push_back({
      {0.228395075, 0.03086419783743},  //
      {0.216049403, 0.03086419783745},  //
      {0.216049403, 0.04320987693746},  //
      {0.228395075, 0.04320987693744},  //
  });
  polys.push_back({
      {0.19135803, 0.04320987693747},   //
      {0.19135803, 0.03086419783748},   //
      {0.179012358, 0.03086419783751},  //
      {0.179012358, 0.04320987693749},  //
  });
  polys.push_back({
      {0.228395075, 0.06790123883752},  //
      {0.216049403, 0.06790123883754},  //
      {0.216049403, 0.08024691793755},  //
      {0.228395075, 0.08024691793753},  //
  });
  polys.push_back({
      {0.19135803, 0.06790123883756},   //
      {0.179012358, 0.06790123883765},  //
      {0.179012358, 0.08024691793758},  //
      {0.19135803, 0.08024691793757},   //
  });
  polys.push_back({
      {0.240740761, 0.0925925973759},  //
      {0.203703731, 0.0925925973762},  //
      {0.203703731, 0.1296296423767},  //
      {0.240740761, 0.1296296423760},  //
  });
  polys.push_back({
      {0.19135803, 0.1049382693761},   //
      {0.179012358, 0.1049382693766},  //
      {0.179012358, 0.1172839483764},  //
      {0.19135803, 0.1172839483763},   //
  });
  polys.push_back({
      {0.228395075, 0.1419753133768},  //
      {0.216049403, 0.1419753133770},  //
      {0.216049403, 0.1543209853773},  //
      {0.228395075, 0.1543209853769},  //
  });
  polys.push_back({
      {0.19135803, 0.1419753133771},   //
      {0.179012358, 0.1419753133777},  //
      {0.179012358, 0.1543209853775},  //
      {0.19135803, 0.1543209853772},   //
  });
  polys.push_back({
      {0.228395075, 0.1790123583774},  //
      {0.216049403, 0.1790123583779},  //
      {0.216049403, 0.191358033780},   //
      {0.228395075, 0.191358033778},   //
  });
  polys.push_back({
      {0.487654328, 0.2530864483786},  //
      {0.475308657, 0.2530864483789},  //
      {0.475308657, 0.2654321193788},  //
      {0.487654328, 0.2654321193787},  //
  });
  polys.push_back({
      {0.450617284, 0.2530864483790},  //
      {0.438271612, 0.2530864483793},  //
      {0.438271612, 0.2654321193792},  //
      {0.450617284, 0.2654321193791},  //
  });
  polys.push_back({
      {0.487654328, 0.2901234633794},  //
      {0.475308657, 0.2901234633796},  //
      {0.475308657, 0.3024691343797},  //
      {0.487654328, 0.3024691343795},  //
  });
  polys.push_back({
      {0.450617284, 0.3024691343798},  //
      {0.450617284, 0.2901234633799},  //
      {0.438271612, 0.2901234633801},  //
      {0.438271612, 0.3024691343800},  //
  });
  polys.push_back({
      {0.413580239, 0.2530864483802},  //
      {0.401234567, 0.2530864483805},  //
      {0.401234567, 0.2654321193804},  //
      {0.413580239, 0.2654321193803},  //
  });
  polys.push_back({
      {0.376543224, 0.2530864483806},  //
      {0.364197552, 0.2530864483829},  //
      {0.364197552, 0.2654321193808},  //
      {0.376543224, 0.2654321193807},  //
  });
  polys.push_back({
      {0.413580239, 0.3024691343809},  //
      {0.413580239, 0.2901234633810},  //
      {0.401234567, 0.2901234633811},  //
      {0.401234567, 0.3024691343812},  //
  });
  polys.push_back({
      {0.487654328, 0.3271605073813},  //
      {0.475308657, 0.3271605073815},  //
      {0.475308657, 0.3395061793818},  //
      {0.487654328, 0.3395061793814},  //
  });
  polys.push_back({
      {0.462962955, 0.3518518813816},  //
      {0.462962955, 0.3148148363817},  //
      {0.42592591, 0.3148148363822},   //
      {0.42592591, 0.3518518813820},   //
  });
  polys.push_back({
      {0.487654328, 0.3641975523819},  //
      {0.475308657, 0.3641975523844},  //
      {0.475308657, 0.3765432243845},  //
      {0.487654328, 0.3765432243843},  //
  });
  polys.push_back({
      {0.450617284, 0.3641975523821},  //
      {0.438271612, 0.3641975523848},  //
      {0.438271612, 0.3765432243847},  //
      {0.450617284, 0.3765432243846},  //
  });
  polys.push_back({
      {0.413580239, 0.3395061793823},  //
      {0.413580239, 0.3271605073824},  //
      {0.401234567, 0.3271605073825},  //
      {0.401234567, 0.3395061793827},  //
  });
  polys.push_back({
      {0.413580239, 0.3641975523828},  //
      {0.401234567, 0.3641975523859},  //
      {0.401234567, 0.3765432243860},  //
      {0.413580239, 0.3765432243858},  //
  });
  polys.push_back({
      {0.339506179, 0.2654321193830},  //
      {0.339506179, 0.2530864483831},  //
      {0.327160507, 0.2530864483833},  //
      {0.327160507, 0.2654321193832},  //
  });
  polys.push_back({
      {0.302469134, 0.2530864483835},  //
      {0.290123463, 0.2530864483837},  //
      {0.290123463, 0.2654321193838},  //
      {0.302469134, 0.2654321193836},  //
  });
  polys.push_back({
      {0.487654328, 0.4012345673849},  //
      {0.475308657, 0.4012345673851},  //
      {0.475308657, 0.4135802393852},  //
      {0.487654328, 0.4135802393850},  //
  });
  polys.push_back({
      {0.450617284, 0.4012345673853},  //
      {0.438271612, 0.4012345673855},  //
      {0.438271612, 0.4135802393856},  //
      {0.450617284, 0.4135802393854},  //
  });
  polys.push_back({
      {0.487654328, 0.4382716123864},  //
      {0.475308657, 0.4382716123866},  //
      {0.475308657, 0.4506172843867},  //
      {0.487654328, 0.4506172843865},  //
  });
  TestPoly(polys, 1771);
}

TEST(Polygon, Woodgrain1) {
  Polygons polys;
  polys.push_back({
      {72.5, -50},   //
      {-72.5, 50},   //
      {-72.5, -50},  //
  });
  polys.push_back({
      {27.7341537, -49.9853325},  //
      {27.4095287, -49.6340637},  //
      {27.9608383, -49.758522},   //
  });
  polys.push_back({
      {-1.56910598, -49.98423},    //
      {-1.58251643, -49.9838409},  //
      {-1.19169295, -48.0386696},  //
      {-1.02179492, -49.9855652},  //
  });
  TestPoly(polys, 12, 0.00145);
}

TEST(Polygon, Woodgrain2) {
  Polygons polys;
  polys.push_back({
      {22.7295628, -49.5001106},   //
      {-21.1059895, -49.9855423},  //
      {-35.0938339, 10.1798563},   //
      {-72.5, -50},                //
      {-12.7149582, -50},          //
  });
  polys.push_back({
      {-40.1850586, -49.9840584},  //
      {-40.2071075, -49.9845123},  //
      {-40.3756485, -49.9855652},  //
      {-40.0327415, -19.857235},   //
  });
  TestPoly(polys, 9, 0.00145);
}

TEST(Polygon, Separate_Quad_And_Tri) {
  Polygons polys;
  polys.push_back({{3, 0}, {3, 1}, {2, 0}});
  polys.push_back({{0.2, 0.8}, {0.8, 0.8}, {1, 1}, {0, 1}});
  TestPoly(polys, 3);
}

TEST(Polygon, Very_Branchy_With_hole) {
  Polygons polys;
  polys.push_back({
      {-12.383395, 191.627597},
      {-45.085282, 233.569954},
      {-49.061051, 241.121459},
      {-55.214156, 240.296329},
      {-59.061922, 241.234954},
      {-60.674552, 242.992235},
      {-61.063571, 242.635239},
      {-61.073658, 243.020877},
      {-61.636008, 243.147031},
      {-62.268425, 240.330262},
      {-61.609029, 240.203785},
      {-59.2187, 240.610881},
      {-55.133, 239.584},
      {-50.566424, 240.105481},
      {-45.687, 232.57},
      {-12.291, 189.141},
      {0, 170},
      {40.7437697, 229.300974},
      {55.7083522, 236.787552},
      {80.252253, 248.42422},
      {79.4563048, 248.689086},
      {56.830082, 238.344589},
      {63.6074538, 247.073547},
      {70.3643318, 251.593951},
      {76.0529064, 252.091121},
      {79.4563, 248.69},
      {75.981, 253.138},
      {69.6668222, 252.703022},
      {62.3134, 248.079},
      {54.8726126, 238.495329},
      {38.62879, 230.54576},
      {0, 180},
  });
  polys.push_back({
      {-61.065936, 242.624693},
      {-59.65022, 241.08199},
      {-61.482003, 240.77002},
  });
  TestPoly(polys, 35);
}
